@@ -22,6 +22,8 @@
 	#define alloca _alloca
 #endif
 
+#ifndef __FreeBSD__
+#ifndef	__NetBSD__
 #ifndef __sun__
 	#ifndef alloca
 	#define alloca __alloca
@@ -29,6 +31,8 @@
 #else
 	#include <alloca.h>
 #endif
+#endif
+#endif
 
 #define wave_alloca alloca
 
