@@ -13,6 +13,7 @@
 #include "lxt.h"
 #include "debug.h"
 #include "bsearch.h"
+#include "strace.h"
 
 #ifdef _MSC_VER
 #define strcasecmp _stricmp
@@ -74,27 +75,43 @@ return(pnt); /* not as many levels as max, so give the full name.. */
  * Add a trace to the display...
  */
 static void AddTrace( Trptr t )
-  {
-   if(default_flags&TR_NUMMASK) t->flags=default_flags;
+{
+if(default_flags&TR_NUMMASK) t->flags=default_flags;
 	else t->flags=(t->flags&TR_NUMMASK)|default_flags;
 
-   if(shift_timebase_default_for_add)
+if(shift_timebase_default_for_add)
 	t->shift=shift_timebase_default_for_add;
 
-    if( traces.first == NULL )
-      {
-	t->next = t->prev = NULL;
-	traces.first = traces.last = t;
-      }
-    else
-      {
-	t->next = NULL;
-	t->prev = traces.last;
-	traces.last->next = t;
-	traces.last = t;
-      }
-    traces.total++;
-  }
+if(!shadow_active)
+	{
+	if( traces.first == NULL )
+		{
+		t->next = t->prev = NULL;
+		traces.first = traces.last = t;
+	      	}
+	    	else
+	      	{
+		t->next = NULL;
+		t->prev = traces.last;
+		traces.last->next = t;
+		traces.last = t;
+	      	}
+	traces.total++;
+	}
+	else	/* hide offscreen */
+	{
+	struct strace *st = calloc_2(1, sizeof(struct strace));
+	st->next = shadow_straces;
+	st->value = shadow_type;
+	st->trace = t;
+
+	st->string = shadow_string;	/* copy string over */
+	shadow_string = NULL;
+
+	shadow_straces = st;
+
+	}
+}
 
 
 /*
@@ -200,7 +217,7 @@ int AddNode(nptr nd, char *aliasname)
     int i;
 
     if(!nd) return(0); /* passed it a null node ptr by mistake */
-    if(nd->mvlfac) import_lxt_trace(nd);
+    if(nd->mvlfac) import_trace(nd);
 
     signalwindow_width_dirty=1;
     
@@ -329,6 +346,16 @@ int AddVector( bvptr vec )
  */
 void FreeTrace(Trptr t)
 {
+if(straces)
+	{
+	struct strace_defer_free *sd = calloc_2(1, sizeof(struct strace_defer_free));
+	sd->next = strace_defer_free_head;
+	sd->defer = t;
+
+	strace_defer_free_head = sd;
+	return;
+	}
+
 if(t->vector)
       	{
       	bvptr bv;
@@ -355,7 +382,7 @@ if(t->vector)
       	}
 	else
 	{
-	if(t->n.nd->expansion)
+	if(t->n.nd && t->n.nd->expansion)
 		{
 		DeleteNode(t->n.nd);
 		}
