@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 1999.
+ * Copyright (c) Tony Bybell 1999-2003.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -10,11 +10,36 @@
 #include "analyzer.h"
 #include "aet.h"
 #include "lxt.h"
+#include "lx2.h"
+#include "lxt2_read.h"
 #include "vcd.h"
 #include "debug.h"
 #include "bsearch.h"
+#include "strace.h"
 #include <stdlib.h>
 
+/*
+ * mvlfac resolution
+ */
+void import_trace(nptr np)
+{
+if(is_lxt)
+	{
+	import_lxt_trace(np);
+	}
+else
+if(is_lx2)
+	{
+	import_lx2_trace(np);
+	}
+else
+	{
+	fprintf(stderr, "Internal error with mvlfac trace handling, exiting.\n");
+	exit(255);
+	}
+}
+
+
 /*
  * turn a Bits structure into a vector with deltas for faster displaying
  */
@@ -112,178 +137,6 @@ return(bitvec);
 }
 
 
-/*
- * Parse a line of the wave file and act accordingly.. 
- * Returns nonzero if trace(s) added.
- */
-int parsewavline(char *w)
-{
-int i, j;
-int len;
-char *prefix, *suffix;
-char *w2;
-nptr nexp;
-
-if(!(len=strlen(w))) return(0);
-if(*(w+len-1)=='\n')
-	{
-	*(w+len-1)=0x00; /* strip newline if present */
-	len--;
-	if(!len) return(0);
-	}
-
-prefix=(char *)wave_alloca(len+1);
-suffix=(char *)wave_alloca(len+1);
-
-w2=w;
-while(1)
-{
-if(isspace(*w2)) { w2++; continue; }
-if(!(*w2)) return(0);	/* no args */
-break;			/* start grabbing chars from here */
-}
-
-sscanf(w2,"%s",prefix);
-if(*w2=='*')
-	{
-	float f;
-	TimeType ttlocal;
-	int which=0;
-
-	zoom_was_explicitly_set=~0;
-	w2++;
-
-	for(;;)
-		{
-		while(*w2==' ') w2++;
-		if(*w2==0) return(~0);
-
-		if(!which) { sscanf(w2,"%f",&f); tims.zoom=(gdouble)f; }
-		else
-		{
-		sscanf(w2,TTFormat,&ttlocal);
-		switch(which)
-			{
-			case 1:  tims.marker=ttlocal; break;
-			default: 
-				if((which-2)<26) named_markers[which-2]=ttlocal; 
-				break;
-			}
-		}
-		which++;
-		w2++;
-		for(;;)
-			{
-			if(*w2==0) return(~0);
-			if(*w2=='\n') return(~0);
-			if(*w2!=' ') w2++; else break;
-			}
-		}
-	}
-else
-if(*w2=='-')
-	{
-	AddBlankTrace((*(w2+1)!=0)?(w2+1):NULL);
-	}
-else
-if(*w2=='>')
-	{
-	char *nptr=(*(w2+1)!=0)?(w2+1):NULL;
-	shift_timebase_default_for_add=nptr?atoi_64(nptr):LLDescriptor(0);
-	}
-else
-if(*w2=='@')
-	{
-	/* handle trace flags */
-	sscanf(w2+1, "%x", &default_flags);
-	return(~0);
-	}
-else
-if(*w2=='+')
-	{
-	/* handle aliasing */
-	struct symbol *s;
-	sscanf(w2+strlen(prefix),"%s",suffix);
-
-	if(suffix[0]=='(')
-		{
-		for(i=1;;i++)
-			{
-			if(suffix[i]==0) return(0);
-			if((suffix[i]==')')&&(suffix[i+1])) {i++; goto aliaspt2; }
-			}
-		return(0);
-
-aliaspt2:
-		s=symfind(suffix+i);
-		nexp = ExtractNodeSingleBit(s->n, atoi(suffix+1));
-		if(nexp)
-			{
-			AddNode(nexp, prefix+1);
-			return(~0);
-			}
-			else
-			{
-			return(0);
-			}		
-		}
-		else
-		{
-		if((s=symfind(suffix)))
-			{
-			AddNode(s->n,prefix+1);
-			return(~0);
-			}
-			else
-			{
-			return(0);
-			}
-		}
-	}
-else
-if(*w2=='#')
-	{
-	/* handle bitvec */
-	bvptr v=NULL;
-	bptr b=NULL;
-
-	w2=w2+strlen(prefix);
-	while(1)
-		{
-		if(isspace(*w2)) { w2++; continue; }
-		if(!(*w2)) return(0);	/* no more args */	
-		break;			/* start grabbing chars from here */
-		}
-
-
-	if((b=makevec(prefix+1,w2)))
-		{
-		if((v=bits2vector(b)))
-			{
-			v->bits=b;	/* only needed for savefile function */
-			AddVector(v);
-			free_2(b->name);
-			b->name=NULL;
-			return(v!=NULL);
-			}
-			else
-			{
-			free_2(b->name);
-			free_2(b);
-			}
-		}
-	
-	return(v!=NULL);
-	}
-else
-	{
-	return(maketraces(w));
-	}
-
-return(0);
-}
-
-
 /*
  * Make solitary traces from wildcarded signals...
  */
@@ -322,16 +175,17 @@ if(!wild_active)	/* short circuit wildcard evaluation with bsearch */
 
 maketracespt2:
 		s=symfind(str+i);
-		nexp = ExtractNodeSingleBit(s->n, atoi(str+1));
-		if(nexp)
+		if(s)
 			{
-			AddNode(nexp, NULL);
-			return(~0);
+			nexp = ExtractNodeSingleBit(s->n, atoi(str+1));
+			if(nexp)
+				{
+				AddNode(nexp, NULL);
+				return(~0);
+				}
 			}
-			else
-			{
-			return(0);
-			}		
+
+		return(0);
 		}
 		else
 		{
@@ -373,6 +227,8 @@ if(len)
 			made=~0;
 			}
 		}
+
+	free_2(wild);
 	}
 
 if(!ch) break;
@@ -439,12 +295,15 @@ if(len)
 					{
 					i++; 
 					s=symfind(wild+i);
-					nexp = ExtractNodeSingleBit(s->n, atoi(wild+1));
-					if(nexp)
+					if(s)
 						{
-						n[nodepnt++]=nexp;
-						if(nodepnt==512) { free_2(wild); goto ifnode; }
-						}		
+						nexp = ExtractNodeSingleBit(s->n, atoi(wild+1));
+						if(nexp)
+							{
+							n[nodepnt++]=nexp;
+							if(nodepnt==512) { free_2(wild); goto ifnode; }
+							}		
+						}
 					break;
 					}
 				}
@@ -486,7 +345,7 @@ if(nodepnt)
 	for(i=0;i<nodepnt;i++)
 		{
 		b->nodes[i]=n[i];
-		if(n[i]->mvlfac) import_lxt_trace(n[i]);
+		if(n[i]->mvlfac) import_trace(n[i]);
 		}
 
 	b->nbits=nodepnt;
@@ -534,7 +393,7 @@ if(nodepnt)
 	for(i=0;i<nodepnt;i++)
 		{
 		b->nodes[i]=n[i];
-		if(n[i]->mvlfac) import_lxt_trace(n[i]);
+		if(n[i]->mvlfac) import_trace(n[i]);
 		}
 
 	b->nbits=nodepnt;
@@ -629,7 +488,7 @@ if(nodepnt)
 	for(i=0;i<nodepnt;i++)
 		{
 		b->nodes[i]=n[i];
-		if(n[i]->mvlfac) import_lxt_trace(n[i]);
+		if(n[i]->mvlfac) import_trace(n[i]);
 		}
 
 	b->nbits=nodepnt;
@@ -817,7 +676,7 @@ if(nodepnt)
 	for(i=0;i<nodepnt;i++)
 		{
 		b->nodes[i]=n[i];
-		if(n[i]->mvlfac) import_lxt_trace(n[i]);
+		if(n[i]->mvlfac) import_trace(n[i]);
 		}
 
 	b->nbits=nodepnt;
@@ -1101,9 +960,11 @@ for(;;)
 }
 #endif
 
-
+#ifndef __linux__
 /* 
- * heapsort algorithm.  this typically outperforms quicksort.
+ * heapsort algorithm.  this typically outperforms quicksort.  note
+ * that glibc will use a modified mergesort if memory is available, so
+ * under linux use the stock qsort instead.
  */
 static struct symbol **hp;
 static void heapify(int i, int heap_size)
@@ -1154,7 +1015,7 @@ for(;;)
         }
 }
 
-void heapsort(struct symbol **a, int num)
+void wave_heapsort(struct symbol **a, int num)
 {
 int i;
 int indx=num-1;
@@ -1185,6 +1046,21 @@ for(;;)
 	}
 }
 
+#else
+
+static int qssigcomp(const void *v1, const void *v2)
+{
+struct symbol *a1 = *((struct symbol **)v1);
+struct symbol *a2 = *((struct symbol **)v2);
+return(sigcmp(a1->name, a2->name));
+}
+
+void wave_heapsort(struct symbol **a, int num)
+{
+qsort(a, num, sizeof(struct symbol *), qssigcomp);
+}
+
+#endif
 
 /*
  * Malloc/Create a name from a range of signals starting from vec_root...currently the single
@@ -1348,7 +1224,7 @@ int offset, len;
 eptr rc=NULL;
 exptr exp;
 
-if(n->mvlfac) import_lxt_trace(n);
+if(n->mvlfac) import_trace(n);
 
 DEBUG(fprintf(stderr, "expanding '%s'\n", n->nname));
 if(!n->ext)
@@ -1530,7 +1406,7 @@ char *nam;
 int offset, len;
 exptr exp;
 
-if(n->mvlfac) import_lxt_trace(n);
+if(n->mvlfac) import_trace(n);
 
 DEBUG(fprintf(stderr, "expanding '%s'\n", n->nname));
 if(!n->ext)
@@ -1713,3 +1589,538 @@ if(n->expansion)
 
 /******************************************************************/
 
+/*
+ * Parse a line of the wave file and act accordingly.. 
+ * Returns nonzero if trace(s) added.
+ */
+int parsewavline(char *w)
+{
+int i, j;
+int len;
+char *prefix, *suffix;
+char *w2;
+nptr nexp;
+
+if(!(len=strlen(w))) return(0);
+if(*(w+len-1)=='\n')
+	{
+	*(w+len-1)=0x00; /* strip newline if present */
+	len--;
+	if(!len) return(0);
+	}
+
+prefix=(char *)wave_alloca(len+1);
+suffix=(char *)wave_alloca(len+1);
+
+w2=w;
+while(1)
+{
+if(isspace(*w2)) { w2++; continue; }
+if(!(*w2)) return(0);	/* no args */
+break;			/* start grabbing chars from here */
+}
+
+sscanf(w2,"%s",prefix);
+if(*w2=='*')
+	{
+	float f;
+	TimeType ttlocal;
+	int which=0;
+
+	zoom_was_explicitly_set=~0;
+	w2++;
+
+	for(;;)
+		{
+		while(*w2==' ') w2++;
+		if(*w2==0) return(~0);
+
+		if(!which) { sscanf(w2,"%f",&f); tims.zoom=(gdouble)f; }
+		else
+		{
+		sscanf(w2,TTFormat,&ttlocal);
+		switch(which)
+			{
+			case 1:  tims.marker=ttlocal; break;
+			default: 
+				if((which-2)<26) named_markers[which-2]=ttlocal; 
+				break;
+			}
+		}
+		which++;
+		w2++;
+		for(;;)
+			{
+			if(*w2==0) return(~0);
+			if(*w2=='\n') return(~0);
+			if(*w2!=' ') w2++; else break;
+			}
+		}
+	}
+else
+if(*w2=='-')
+	{
+	AddBlankTrace((*(w2+1)!=0)?(w2+1):NULL);
+	}
+else
+if(*w2=='>')
+	{
+	char *nptr=(*(w2+1)!=0)?(w2+1):NULL;
+	shift_timebase_default_for_add=nptr?atoi_64(nptr):LLDescriptor(0);
+	}
+else
+if(*w2=='@')
+	{
+	/* handle trace flags */
+	sscanf(w2+1, "%x", &default_flags);
+	return(~0);
+	}
+else
+if(*w2=='+')
+	{
+	/* handle aliasing */
+	struct symbol *s;
+	sscanf(w2+strlen(prefix),"%s",suffix);
+
+	if(suffix[0]=='(')
+		{
+		for(i=1;;i++)
+			{
+			if(suffix[i]==0) return(0);
+			if((suffix[i]==')')&&(suffix[i+1])) {i++; goto aliaspt2; }
+			}
+		return(0);
+
+aliaspt2:
+		s=symfind(suffix+i);
+		nexp = ExtractNodeSingleBit(s->n, atoi(suffix+1));
+		if(nexp)
+			{
+			AddNode(nexp, prefix+1);
+			return(~0);
+			}
+			else
+			{
+			return(0);
+			}		
+		}
+		else
+		{
+		if((s=symfind(suffix)))
+			{
+			AddNode(s->n,prefix+1);
+			return(~0);
+			}
+			else
+			{
+			return(0);
+			}
+		}
+	}
+else
+if(*w2=='#')
+	{
+	/* handle bitvec */
+	bvptr v=NULL;
+	bptr b=NULL;
+
+	w2=w2+strlen(prefix);
+	while(1)
+		{
+		if(isspace(*w2)) { w2++; continue; }
+		if(!(*w2)) return(0);	/* no more args */	
+		break;			/* start grabbing chars from here */
+		}
+
+
+	if((b=makevec(prefix+1,w2)))
+		{
+		if((v=bits2vector(b)))
+			{
+			v->bits=b;	/* only needed for savefile function */
+			AddVector(v);
+			free_2(b->name);
+			b->name=NULL;
+			return(v!=NULL);
+			}
+			else
+			{
+			free_2(b->name);
+			free_2(b);
+			}
+		}
+	
+	return(v!=NULL);
+	}
+else
+if(*w2=='!')
+	{
+	/* fill logical_mutex */
+	int i;
+	char ch;
+
+	for(i=0;i<6;i++)
+		{
+		ch = *(w2+i+1);
+		if(ch != 0)
+			{
+			if(ch=='!')
+				{
+				shadow_active = 0;
+				return(~0);
+				}
+
+			if((!i)&&(shadow_straces))
+				{
+				delete_strace_context();
+				}
+
+			shadow_logical_mutex[i] = (ch & 1);
+			}
+			else	/* in case of short read */
+			{
+			shadow_logical_mutex[i] = 0;
+			}
+		}
+
+	shadow_active = 1;
+	return(~0);
+	}
+	else
+if(*w2=='?')
+	{
+	/* fill st->type */
+	if(*(w2+1)=='\"')
+		{
+		int len = strlen(w2+2);
+		if(shadow_string) free_2(shadow_string);
+		shadow_string=NULL;
+
+		if(len)
+			{
+			shadow_string = malloc_2(len+1);		
+			strcpy(shadow_string, w2+2);
+			}
+
+		shadow_type = ST_STRING;
+		}
+		else
+		{
+		int hex;
+		sscanf(w2+1, "%x", &hex);	
+		shadow_type = hex;
+		}
+
+	return(~0);
+	}
+	else
+	{
+	return(maketraces(w));
+	}
+
+return(0);
+}
+
+
+/****************/
+/* LX2 variants */
+/****************/
+
+/*
+ * Make solitary traces from wildcarded signals...
+ */
+void maketraces_lx2(char *str)
+{
+char *pnt, *wild;
+char ch, wild_active=0;
+int len;
+int i;
+
+pnt=str;
+while((ch=*pnt))
+	{
+	if(ch=='*') 
+		{
+		wild_active=1;
+		break;
+		}
+	pnt++;
+	}
+
+if(!wild_active)	/* short circuit wildcard evaluation with bsearch */
+	{
+	struct symbol *s;
+	nptr nexp;
+
+	if(str[0]=='(')
+		{
+		for(i=1;;i++)
+			{
+			if(str[i]==0) return;
+			if((str[i]==')')&&(str[i+1])) {i++; goto maketracespt2; }
+			}
+		return;
+
+maketracespt2:
+		if((s=symfind(str+i)))
+			{
+			lx2_set_fac_process_mask(s->n);
+			}
+		return;
+		}
+		else
+		{
+		if((s=symfind(str)))
+			{
+			lx2_set_fac_process_mask(s->n);
+			}
+		return;
+		}
+	}
+
+while(1)
+{
+pnt=str;
+len=0;
+
+while(1)
+	{
+	ch=*pnt++;
+	if(isspace(ch)||(!ch)) break;
+	len++;
+	}
+
+if(len)
+	{
+	wild=(char *)calloc_2(1,len+1);
+	memcpy(wild,str,len);
+	wave_regex_compile(wild);
+
+	for(i=0;i<numfacs;i++)
+		{
+		if(wave_regex_match(facs[i]->name))
+			{
+			lx2_set_fac_process_mask(facs[i]->n);
+			}
+		}
+
+	free_2(wild);
+	}
+
+if(!ch) break;
+str=pnt;
+}
+return;
+}
+
+
+/*
+ * Create a vector from wildcarded signals...
+ */
+void makevec_lx2(char *str)
+{
+char *pnt, *pnt2, *wild=NULL;
+char ch, ch2, wild_active;
+int len, nodepnt=0;
+int i;
+
+while(1)
+{
+pnt=str;
+len=0;
+
+while(1)
+	{
+	ch=*pnt++;
+	if(isspace(ch)||(!ch)) break;
+	len++;
+	}
+
+if(len)
+	{
+	wild=(char *)calloc_2(1,len+1);
+	memcpy(wild,str,len);
+
+	DEBUG(printf("WILD: %s\n",wild));
+
+	wild_active=0;
+	pnt2=wild;
+	while((ch2=*pnt2))
+		{
+		if(ch2=='*') 
+			{
+			wild_active=1;
+			break;
+			}
+		pnt2++;
+		}
+
+	if(!wild_active)	/* short circuit wildcard evaluation with bsearch */
+		{
+		struct symbol *s;
+		if(wild[0]=='(')
+			{
+			nptr nexp;
+			
+			for(i=1;;i++)
+				{
+				if(wild[i]==0) break;
+				if((wild[i]==')')&&(wild[i+1])) 
+					{
+					i++; 
+					s=symfind(wild+i);
+					if(s)
+						{
+						lx2_set_fac_process_mask(s->n);
+						}
+					break;
+					}
+				}
+			}
+			else
+			{
+			if((s=symfind(wild)))	
+				{
+				lx2_set_fac_process_mask(s->n);
+				}
+			}
+		}
+		else
+		{
+		wave_regex_compile(wild);
+		for(i=numfacs-1;i>=0;i--)	/* to keep vectors in little endian hi..lo order */
+			{
+			if(wave_regex_match(facs[i]->name))
+				{
+				lx2_set_fac_process_mask(facs[i]->n);
+				}
+			}
+		}
+	free_2(wild);
+	}
+
+if(!ch) break;
+str=pnt;
+}
+}
+
+
+/*
+ * Parse a line of the wave file and act accordingly.. 
+ * Returns nonzero if trace(s) added.
+ */
+void parsewavline_lx2(char *w)
+{
+int i, j;
+int len;
+char *prefix, *suffix;
+char *w2;
+nptr nexp;
+
+if(!(len=strlen(w))) return;
+if(*(w+len-1)=='\n')
+	{
+	*(w+len-1)=0x00; /* strip newline if present */
+	len--;
+	if(!len) return;
+	}
+
+prefix=(char *)wave_alloca(len+1);
+suffix=(char *)wave_alloca(len+1);
+
+w2=w;
+while(1)
+{
+if(isspace(*w2)) { w2++; continue; }
+if(!(*w2)) return;	/* no args */
+break;			/* start grabbing chars from here */
+}
+
+sscanf(w2,"%s",prefix);
+if(*w2=='*')
+	{
+	}
+else
+if(*w2=='-')
+	{
+	}
+else
+if(*w2=='>')
+	{
+	}
+else
+if(*w2=='@')
+	{
+	}
+else
+if(*w2=='+')
+	{
+	/* handle aliasing */
+	struct symbol *s;
+	sscanf(w2+strlen(prefix),"%s",suffix);
+
+	if(suffix[0]=='(')
+		{
+		for(i=1;;i++)
+			{
+			if(suffix[i]==0) return;
+			if((suffix[i]==')')&&(suffix[i+1])) {i++; goto aliaspt2; }
+			}
+		return;
+
+aliaspt2:
+		s=symfind(suffix+i);
+		if(s)
+			{
+			lx2_set_fac_process_mask(s->n);
+			}
+		return;
+		}
+		else
+		{
+		if((s=symfind(suffix)))
+			{
+			lx2_set_fac_process_mask(s->n);
+			}
+		return;
+		}
+	}
+else
+if(*w2=='#')
+	{
+	/* handle bitvec */
+	bvptr v=NULL;
+	bptr b=NULL;
+
+	w2=w2+strlen(prefix);
+	while(1)
+		{
+		if(isspace(*w2)) { w2++; continue; }
+		if(!(*w2)) return;	/* no more args */	
+		break;			/* start grabbing chars from here */
+		}
+
+	makevec_lx2(w2);
+	return;
+	}
+else
+if(*w2=='!')
+	{
+	}
+	else
+if(*w2=='?')
+	{
+	}
+	else
+	{
+	maketraces_lx2(w);
+	}
+
+return;
+}
+
+
+/****************/
+/* LX2 variants */
+/****************/
+
