@@ -114,6 +114,44 @@ if(i)
 }
 
 
+void reformat_time_blackout(char *buf, TimeType val, char dim)
+{
+char *pnt;
+int i, offset;
+struct blackout_region_t *bt = blackout_regions;
+char blackout = ' ';
+
+while(bt)
+	{
+	if((val>=bt->bstart)&&(val<bt->bend))
+		{
+		blackout = '*';
+		break;
+		}
+
+	bt=bt->next;
+	}
+
+pnt=strchr(time_prefix, (int)dim);
+if(pnt) { offset=pnt-time_prefix; } else offset=0;
+
+for(i=offset; i>0; i--)
+	{
+	if(val%1000) break;
+	val=val/1000;
+	}
+
+if(i)
+	{
+	sprintf(buf, TTFormat"%c%cs", val, blackout, time_prefix[i]);
+	}
+	else
+	{
+	sprintf(buf, TTFormat"%csec", val, blackout);
+	}
+}
+
+
 void update_markertime(TimeType val)
 {
 if(!use_maxtime_display)
@@ -157,7 +195,7 @@ if(use_maxtime_display)
 void update_currenttime(TimeType val)
 {
 currenttime=val;
-reformat_time(curtext, val, time_dimension);
+reformat_time_blackout(curtext, val, time_dimension);
 gtk_label_set(GTK_LABEL(curtimewid), curtext);
 }
 
