@@ -16,7 +16,15 @@
 #include <math.h>
 #include "analyzer.h"
 
-#define WAVE_VERSION_INFO "GTKWave Analyzer v1.3.11 (w)1999-2002 BSI"
+#define WAVE_VERSION_INFO "GTKWave Analyzer v1.3.34 (w)1999-2003 BSI"
+
+struct blackout_region_t
+{
+struct blackout_region_t *next;
+TimeType bstart, bend;
+};
+
+extern struct blackout_region_t *blackout_regions;
 
 extern char is_vcd;
 extern char is_lxt;
@@ -41,6 +49,7 @@ extern char left_justify_sigs;	     /* when enabled, signals are left, rather th
 extern int  ps_maxveclen;	     /* maximum vector length that is printable in signal window (includes = and base symbol) */
 extern char zoom_pow10_snap;	     /* forces divisible by 10 timescale no matter what the zoom is */
 extern char force_toolbars;          /* forces menu+top buttons to be rendered as toolbars */
+extern int  cursor_snap;	     /* makes marker jump to nearest transition (default = 0 pixels) */
 
 extern gdouble page_divisor;	     /* allows fractional page scrolls */
 
