@@ -6,6 +6,7 @@
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  */
+#include "gtk12compat.h"
 #include "currenttime.h"
 
 char *fontname_signals=NULL;
@@ -41,7 +42,7 @@ if((fontname_signals)&&(strlen(fontname_signals)))
 
 if(!signalfont)
         {  
-        signalfont=wavearea->style->font;
+        signalfont=WAVE_GTK_SIGFONT;
         }
 
 fontheight=(signalfont->ascent+signalfont->descent)+4;
@@ -65,7 +66,7 @@ if((fontname_waves)&&(strlen(fontname_waves)))
 
 if(!wavefont)
         {  
-        wavefont=wavefont_smaller=wavearea->style->font;
+        wavefont=wavefont_smaller=WAVE_GTK_WAVEFONT;
         }
 
 if(signalfont->ascent<wavefont->ascent)
