@@ -1,10 +1,23 @@
-/* 
- * Copyright (c) Tony Bybell 2001.
+/*
+ * Copyright (c) 2001 Tony Bybell.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation 
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
+ * and/or sell copies of the Software, and to permit persons to whom the   
+ * Software is furnished to do so, subject to the following conditions: 
+ *
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,  
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL   
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING   
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  */
 
 #include <stdio.h>
@@ -56,33 +69,12 @@ static int tableposition=0;
 static int trace_start=0;
 static FILE *thandle;
 static int fsiz=0;
-static struct namehier *nhold=NULL;
 static off_t f_len;
 int mesa=0;
 int version;
 
 /****************************************************************************/
 
-/*
- * gtkwave chokes on hierarchies with $ prefixes
- */
-static char undollar_buf[8192];
-static char *undollar(char *s)
-{
-char *p=undollar_buf;
-strcpy(p,s);
-
-while(*p)
-	{
-	if(*p=='$') *p='_';
-	p++;
-	}
-
-return(undollar_buf);
-}
-
-/****************************************************************************/
-
 /*
  * reconstruct 8/16/24/32 bits out of the aet's representation
  * of a big-endian integer.  this can be optimized for 
@@ -207,7 +199,6 @@ int l, r;
 unsigned int largest;
 int heap_size=numfacs;
 unsigned int t;
-int start=i;
 
 for(;;)
 	{
@@ -336,7 +327,7 @@ for(;;)
 				break;
 			default:
 				fprintf(stderr, MVLHDR"Unknown opcode %02x found in reverse traversal at %08x (sym=%s), exiting.", 
-					opcode, pos, facs[lcindex[0]].name,opcode);			
+					opcode, pos, facs[lcindex[0]].name);			
 				exit(0);
 			}
 		heapify(0);				
@@ -629,7 +620,6 @@ int v, j, k, nbits, l;
 int skip=0;
 char *pnt;
 char buf[65537];
-char first, second;
 
 l=nbits=facs[which].len;
 
@@ -779,12 +769,11 @@ if(l==1)
 /*
  * generate the LXT file 
  */
-static void output_lxt(char *fnam, int clock_compress)
+static void output_lxt(char *fnam, int clock_compress, int change_compress)
 {
 int i;
-char buf[4096];
 unsigned int which, pos;
-unsigned int tmval, tmvalchk;
+unsigned int tmval=~0, tmvalchk;
 struct lt_trace *lt;
 
 fprintf(stderr, MVLHDR"Generating LXT file '%s'...\n", fnam);
@@ -802,6 +791,7 @@ for(i=0;i<numfacs;i++)
 	}
 
 if(clock_compress) lt_set_clock_compress(lt);
+if(change_compress) lt_set_chg_compress(lt);
 lt_set_timescale(lt, -9);
 lt_set_time(lt, first_cycle);
 lt_set_initial_value(lt, '0');
@@ -835,14 +825,20 @@ lt_close(lt);
 int main(int argc, char **argv)
 {
 int fd;
-int doclock;
+int i, doclock=0, dochg=0;
 
 if(argc<3)
         {
-        fprintf(stderr, "Usage:\n------\n%s filename.aet filename.lxt [-clockpack]\n", argv[0]);
+        fprintf(stderr, "Usage:\n------\n%s filename.aet filename.lxt [-clockpack][-chgpack]\n", argv[0]);
         exit(0);
         }
-doclock =  (argc>=4)&&(!strcmp(argv[3], "-clockpack"));
+
+for(i=3;i<argc;i++)
+        {
+        if(!strcmp(argv[i], "-clockpack")) doclock = 1;
+        else if(!strcmp(argv[i], "-chgpack")) dochg = 1;
+        }
+
 
 fd=open(argv[1], O_RDONLY);
 if(fd<0)
@@ -884,7 +880,7 @@ build_facs2(f_len);
 sync_timetable();
 
 iter_backwards();
-output_lxt(argv[2], doclock);
+output_lxt(argv[2], doclock, dochg);
 
 munmap(mm, f_len); mm=NULL;
 fprintf(stderr, MVLHDR"Finished, exiting.\n");
