@@ -1,10 +1,23 @@
-/* 
- * Copyright (c) Tony Bybell 1999.
+/*
+ * Copyright (c) 2001 Tony Bybell.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation 
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
+ * and/or sell copies of the Software, and to permit persons to whom the   
+ * Software is furnished to do so, subject to the following conditions: 
+ *
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,  
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL   
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING   
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  */
 
 /*
@@ -41,7 +54,6 @@ static char vcd_is_compressed=0;
 
 static void add_histent(TimeType time, struct Node *n, char ch, int regadd, char *vector);
 static void add_tail_histents(void);
-static void vcd_build_symbols(void);
 static void evcd_strcpy(char *dst, char *src);
 
 static int vcdlineno=1;
@@ -88,6 +100,7 @@ static int yylen=0, yylen_cache=0;
 
 static struct vcdsymbol *vcdsymroot=NULL, *vcdsymcurr=NULL;
 static struct vcdsymbol **sorted=NULL;
+static struct vcdsymbol **indexed=NULL;
 
 enum VarTypes { V_EVENT, V_PARAMETER,
 		V_INTEGER, V_REAL, V_REG, V_SUPPLY0,
@@ -109,6 +122,30 @@ static int numsyms=0;
 
 static struct queuedevent *queuedevents=NULL;
 
+/******************************************************************/
+
+static unsigned int vcd_minid = ~0;
+static unsigned int vcd_maxid = 0;
+
+static unsigned int vcdid_hash(char *s, int len)
+{
+unsigned int val=0;
+int i;
+
+s+=(len-1);
+
+for(i=0;i<len;i++)
+        {
+        val *= 95;                              /* was 94 but XL uses '!' as right hand side chars which act as leading zeros */
+        val += (((unsigned char)*s) - 32);      /* was 33 but XL ... */                                
+        s--;
+        }
+
+return(val);
+}
+
+
+
 /******************************************************************/
 
 /*
@@ -129,11 +166,20 @@ return(strcmp(v1, v2->id));
 /*
  * actual bsearch
  */
-static struct vcdsymbol *bsearch_vcd(char *key)
+static struct vcdsymbol *bsearch_vcd(char *key, int len)
 {
 struct vcdsymbol **v;
 struct vcdsymbol *t;
 
+if(indexed)
+	{
+	unsigned int hsh = vcdid_hash(key, len);
+	if((hsh>=vcd_minid)&&(hsh<=vcd_maxid))
+		{
+		return(indexed[hsh-vcd_minid]);
+		}
+	}
+
 v=(struct vcdsymbol **)bsearch(key, sorted, numsyms,
         sizeof(struct vcdsymbol *), vcdsymbsearchcompare);
 
@@ -181,18 +227,37 @@ static void create_sorted_table(void)
 {
 struct vcdsymbol *v;
 struct vcdsymbol **pnt;
+unsigned int vcd_distance;
 
 if(numsyms)
 	{
-	pnt=sorted=(struct vcdsymbol **)calloc_2(numsyms, sizeof(struct vcdsymbol *));
-	v=vcdsymroot;
-	while(v)
-		{
-		*(pnt++)=v;
-		v=v->next;
-		}
+	vcd_distance = vcd_maxid - vcd_minid + 1;
 
-	qsort(sorted, numsyms, sizeof(struct vcdsymbol *), vcdsymcompare);
+	if(vcd_distance <= 2 * 1024 * 1024)
+		{
+		indexed = (struct vcdsymbol **)calloc_2(vcd_distance, sizeof(struct vcdsymbol *));
+
+		printf("%d symbols span ID range of %d, using indexing...\n", numsyms, vcd_distance);
+
+		v=vcdsymroot;
+		while(v)
+			{
+			if(!indexed[v->nid - vcd_minid]) indexed[v->nid - vcd_minid] = v;
+			v=v->next;
+			}
+		}
+		else
+		{
+		pnt=sorted=(struct vcdsymbol **)calloc_2(numsyms, sizeof(struct vcdsymbol *));
+		v=vcdsymroot;
+		while(v)
+			{
+			*(pnt++)=v;
+			v=v->next;
+			}
+	
+		qsort(sorted, numsyms, sizeof(struct vcdsymbol *), vcdsymcompare);
+		}
 	}
 }
 
@@ -508,6 +573,11 @@ while(s)
 	s=s->next;
 	}
 
+if(slisthier)
+        {
+        free_2(slisthier);
+        }
+
 slisthier=(char *)malloc_2((slisthier_len=len)+1);
 s=slistroot; len=0;
 while(s)
@@ -564,7 +634,7 @@ switch(yytext[0])
 	case 'Z':
 		if(yylen>1)
 			{
-			v=bsearch_vcd(yytext+1);	
+			v=bsearch_vcd(yytext+1, yylen-1);	
 			if(!v)
 				{
 				fprintf(stderr,"Near line %d, Unknown VCD identifier: '%s'\n",vcdlineno,yytext+1);
@@ -618,7 +688,7 @@ switch(yytext[0])
 		vlen=yylen-1;
 
 		get_strtoken();
-		v=bsearch_vcd(yytext);	
+		v=bsearch_vcd(yytext, yylen);	
 		if(!v)
 			{
 			fprintf(stderr,"Near line %d, Unknown identifier: '%s'\n",vcdlineno, yytext);
@@ -689,7 +759,7 @@ switch(yytext[0])
 		get_strtoken();	/* throw away 0_strength_component */
 		get_strtoken(); /* throw away 0_strength_component */
 		get_strtoken(); /* this is the id                  */
-		v=bsearch_vcd(yytext);	
+		v=bsearch_vcd(yytext, yylen);	
 		if(!v)
 			{
 			fprintf(stderr,"Near line %d, Unknown identifier: '%s'\n",vcdlineno, yytext);
@@ -758,7 +828,7 @@ switch(yytext[0])
 		sscanf(yytext+1,"%lg",d);
 		
 		get_strtoken();
-		v=bsearch_vcd(yytext);	
+		v=bsearch_vcd(yytext, yylen);	
 		if(!v)
 			{
 			fprintf(stderr,"Near line %d, Unknown identifier: '%s'\n",vcdlineno, yytext);
@@ -774,6 +844,13 @@ switch(yytext[0])
 
 		break;
 		}
+
+	case 's':
+	case 'S':
+		{
+		get_strtoken();	/* simply skip for now */
+		break;
+		}
 	}
 
 }
@@ -807,7 +884,7 @@ while((ch=*src))
 }
 
 
-static void vcd_parse(void)
+static void vcd_parse(int linear)
 {
 int tok;
 
@@ -829,11 +906,18 @@ for(;;)
 			int vtok;
 			int i;
 			char prefix=' ';
+			int timelogadjust = 0;
 
 			vtok=get_token();
 			if((vtok==T_END)||(vtok==T_EOF)) break;
 			time_scale=atoi_64(yytext);
 			if(!time_scale) time_scale=1;
+			switch(time_scale)
+				{
+				case 10:	timelogadjust = +1;	break;
+				case 100:	timelogadjust = +2;	break;
+				}
+
 			for(i=0;i<yylen;i++)
 				{
 				if((yytext[i]<'0')||(yytext[i]>'9'))
@@ -851,14 +935,14 @@ for(;;)
 			switch(prefix)
 				{
 				case 's':
-				case ' ': lt_set_timescale(lt, 0); break;
-				case 'm': lt_set_timescale(lt, -3); break;
-				case 'u': lt_set_timescale(lt, -6); break;
-				case 'n': lt_set_timescale(lt, -9); break;
-				case 'p': lt_set_timescale(lt, -12); break;
-				case 'f': lt_set_timescale(lt, -15); break;
+				case ' ': lt_set_timescale(lt,  0+timelogadjust); break;
+				case 'm': lt_set_timescale(lt, -3+timelogadjust); break;
+				case 'u': lt_set_timescale(lt, -6+timelogadjust); break;
+				case 'n': lt_set_timescale(lt, -9+timelogadjust); break;
+				case 'p': lt_set_timescale(lt, -12+timelogadjust); break;
+				case 'f': lt_set_timescale(lt, -15+timelogadjust); break;
 				default:	/* unknown */
-					  lt_set_timescale(lt, -9); break;
+					  lt_set_timescale(lt, -9+timelogadjust); break;
 				}
 
 			sync_end(NULL);
@@ -980,6 +1064,10 @@ for(;;)
 				if(vtok==V_END) goto err;
 				v->id=(char *)malloc_2(yylen+1);
 				strcpy(v->id, yytext);
+				v->nid=vcdid_hash(yytext,yylen);
+
+				if(v->nid < vcd_minid) vcd_minid = v->nid;
+				if(v->nid > vcd_maxid) vcd_maxid = v->nid;
 
 				vtok=get_vartoken();
 				if(vtok!=V_STRING) goto err;
@@ -1006,6 +1094,10 @@ for(;;)
 				if(vtok==V_END) goto err;
 				v->id=(char *)malloc_2(yylen+1);
 				strcpy(v->id, yytext);
+				v->nid=vcdid_hash(yytext,yylen);
+
+				if(v->nid < vcd_minid) vcd_minid = v->nid;
+				if(v->nid > vcd_maxid) vcd_maxid = v->nid;
 
 				vtok=get_vartoken();
 				if(vtok!=V_STRING) goto err;
@@ -1152,11 +1244,13 @@ for(;;)
 				{
 				header_over=1;	/* do symbol table management here */
 				create_sorted_table();
-				if(!sorted)
+				if((!sorted)&&(!indexed))
 					{
 					fprintf(stderr, "No symbols in VCD file..nothing to do!\n");
 					exit(1);
 					}
+
+				if(linear)  lt_set_no_interlace(lt);
 				}
 			break;
 		case T_STRING:
@@ -1174,7 +1268,7 @@ for(;;)
 						}
                                         current_time=time;
                                         if(end_time<time) end_time=time;        /* in case of malformed vcd files */
-					lt_set_time(lt, current_time);
+					lt_set_time64(lt, current_time);
 					DEBUG(fprintf(stderr,"#"TTFormat"\n",time));
 					}
 					else
@@ -1222,7 +1316,6 @@ for(;;)
 void add_histent(TimeType time, struct Node *n, char ch, int regadd, char *vector)
 {
 struct HistEnt *he;
-char heval;
 
 if(!vector)
 {
@@ -1305,8 +1398,6 @@ if(ch=='g')	/* real number */
 
 static void add_tail_histents(void)
 {
-int i,j;
-
 /* dump out any pending events 1st */
 struct queuedevent *q;
 q=queuedevents;
@@ -1327,214 +1418,6 @@ while(q)
 
 /*******************************************************************************/
 
-static void vcd_build_symbols(void)
-{
-int i,j;
-int max_slen=-1;
-int duphier=0;
-
-for(i=0;i<numsyms;i++)
-	{
-	struct vcdsymbol *v, *vprime;
-	int msi;
-	int delta;
-
-		{
-		char *str;
-		int slen;
-		int substnode;
-
-		v=sorted[i];
-		msi=v->msi;
-		delta=((v->lsi-v->msi)<0)?-1:1;
-		substnode=0;
-
-                slen=strlen(v->name);
-                str=(slen>max_slen)?(wave_alloca((max_slen=slen)+32)):(str); /* more than enough */
-		strcpy(str,v->name);
-
-		if(v->msi>=0)
-			{
-			strcpy(str+slen,vcd_hier_delimeter);
-			slen++;
-			}
-
-		if((vprime=bsearch_vcd(v->id))!=v) /* hash mish means dup net */
-			{
-			if(v->size!=vprime->size)
-				{
-				fprintf(stderr,"ERROR: Duplicate IDs with differing width: %s %s\n", v->name, vprime->name);
-				}
-				else
-				{
-				substnode=1;
-
-				v->ltsym->flags=LT_SYM_F_ALIAS;	/* alias to our other node that bsearch found */
-				v->ltsym->aliased_to=vprime->ltsym;
-				}
-			}
-
-		if(((v->size==1)||(!atomic_vectors))&&(v->vartype!=V_REAL))
-		for(j=0;j<v->size;j++)
-			{
-			if(v->msi>=0) 
-				{
-				if(!vcd_explicit_zero_subscripts)
-					sprintf(str+slen,"%d",msi);
-					else
-					sprintf(str+slen-1,"[%d]",msi);
-				}
-
-                        if(symfind(str))
-                                {
-                                char *dupfix=(char *)malloc_2(max_slen+32);
-
-                                DEBUG(fprintf(stderr,"Warning: %s is a duplicate VCD symbol.\n",str));
-
-                                do sprintf(dupfix, "$DUP%d%s%s", duphier++, vcd_hier_delimeter, str);
-                                        while(symfind(dupfix));
-
-                                strcpy(str, dupfix);
-                                free_2(dupfix);
-                                duphier=0; /* reset for next duplicate resolution */
-                                }
-                                /* fallthrough */
-				{
-				struct symbol *s;
-
-				s=symadd(str,hash(str));
-
-				s->n=v->narray[j];
-				if(substnode)
-					{
-					struct Node *n, *n2;
-
-					n=s->n;
-					n2=vprime->narray[j];
-					/* nname stays same */
-					n->head=n2->head;
-					n->curr=n2->curr;
-					/* harray calculated later */
-					n->numhist=n2->numhist;
-
-					n->substnode=n2;
-
-					if(!n2->substhead)
-						{
-						n2->substhead=n;
-						}
-						else
-						{
-						while(n2->substhead) n2=n2->substhead;
-						n2->substhead=n;
-						}
-					}
-
-				s->n->nname=s->name;
-				s->h=s->n->curr;
-				if(!firstnode)
-					{
-					firstnode=curnode=s;
-					}
-					else
-					{
-					curnode->nextinaet=s;
-					curnode=s;
-					}
-
-				numfacs++;
-				DEBUG(fprintf(stderr,"Added: %s\n",str));
-				}
-			msi+=delta;
-			}
-			else	/* atomic vector */
-			{
-			if(v->vartype!=V_REAL)
-				{
-				sprintf(str+slen-1,"[%d:%d]",v->msi,v->lsi);
-				}
-				else
-				{
-				*(str+slen-1)=0;
-				}
-
-                        if(symfind(str))
-                                {
-                                char *dupfix=(char *)malloc_2(max_slen+32);
-
-                                DEBUG(fprintf(stderr,"Warning: %s is a duplicate VCD symbol.\n",str));
-
-                                do sprintf(dupfix, "$DUP%d%s%s", duphier++, vcd_hier_delimeter, str);
-                                        while(symfind(dupfix));
-
-                                strcpy(str, dupfix);
-                                free_2(dupfix);
-                                duphier=0; /* reset for next duplicate resolution */
-                                }
-                                /* fallthrough */
-				{
-				struct symbol *s;
-
-				s=symadd(str,hash(str));
-
-				s->n=v->narray[0];
-				if(substnode)
-					{
-					struct Node *n, *n2;
-
-					n=s->n;
-					n2=vprime->narray[0];
-					/* nname stays same */
-					n->head=n2->head;
-					n->curr=n2->curr;
-					/* harray calculated later */
-					n->numhist=n2->numhist;
-					n->ext=n2->ext;
-
-					n->substnode=n2;
-
-					if(!n2->substhead)
-						{
-						n2->substhead=n;
-						}
-						else
-						{
-						while(n2->substhead) n2=n2->substhead;
-						n2->substhead=n;
-						}
-					}
-					else
-					{
-					struct ExtNode *en;
-					en=(struct ExtNode *)malloc_2(sizeof(struct ExtNode));
-					en->msi=v->msi;
-					en->lsi=v->lsi;
-
-					s->n->ext=en;
-					}
-
-				s->n->nname=s->name;
-				s->h=s->n->curr;
-				if(!firstnode)
-					{
-					firstnode=curnode=s;
-					}
-					else
-					{
-					curnode->nextinaet=s;
-					curnode=s;
-					}
-
-				numfacs++;
-				DEBUG(fprintf(stderr,"Added: %s\n",str));
-				}
-			}
-		}
-	}
-}
-
-/*******************************************************************************/
-
 void vcd_sortfacs(void)
 {
 int i;
@@ -1544,7 +1427,6 @@ curnode=firstnode;
 for(i=0;i<numfacs;i++)
         {
         char *subst, ch;
-        int len;
                  
         facs[i]=curnode;
 	subst=curnode->name;
@@ -1617,8 +1499,10 @@ printf("\n[%d] total facilities: [%d] facilit%s defined, [%d] facilit%s undefine
 
 /*******************************************************************************/
 
-TimeType vcd_main(char *fname, char *lxname, int dostats, int doclock)
+TimeType vcd_main(char *fname, char *lxname, int dostats, int doclock, int dochg, int dodict, int linear)
 {
+struct vcdsymbol *v, *v2;
+
 vcd_hier_delimeter[0]=hier_delimeter;
 
 errno=0;	/* reset in case it's set for some reason */
@@ -1665,20 +1549,20 @@ if(!lt)
 	}
 
 if(doclock) lt_set_clock_compress(lt);
+if(dochg)   lt_set_chg_compress(lt);
+if(dodict)  lt_set_dict_compress(lt, dodict);
 lt_set_initial_value(lt, 'x');
 lt_symbol_bracket_stripping(lt, 1);	/* this is intentional */
 
 sym=(struct symbol **)calloc_2(SYMPRIME,sizeof(struct symbol *));
 printf("\nConverting VCD File '%s' to LXT file '%s'...\n\n",(vcd_handle!=stdin)?fname:"from stdin", lxname);
 build_slisthier();
-vcd_parse();
+vcd_parse(linear);
 
 add_tail_histents();
 
 printf("["TTFormat"] start time.\n["TTFormat"] end time.\n\n", start_time, end_time);
 
-vcd_build_symbols();
-
 lt_close(lt);
 lt=NULL;
 
@@ -1693,6 +1577,57 @@ if((min_time==max_time)||(max_time==0))
         exit(1);
         }
 
+if(vcd_handle!=stdin)
+        {
+        fclose(vcd_handle);
+        vcd_handle=NULL;
+        }
+
+free(yytext); yytext=NULL;
+if(indexed) { free(indexed); indexed=NULL; }
+if(sorted) { free(sorted); sorted=NULL; }
+
+v=vcdsymroot;
+while(v) 
+        {
+        if(v->name) { free(v->name); v->name=NULL; }
+        if(v->id) { free(v->id); v->id=NULL; }
+        if(v->value) { free(v->value); v->value=NULL; }
+        
+        if(v->narray)
+                {
+                int i;
+                for(i=0;i<v->size;i++)
+                        {
+                        struct HistEnt *h1, *h2;
+         
+                        if((h1 = v->narray[i]->head.next))
+                                {
+                                h1 = v->narray[i]->head.next;
+                                while(h1)
+                                        {
+                                        h2 = h1->next;
+                                        free(h1);
+                                        h1 = h2;
+                                        }
+                                }
+
+                        free(v->narray[i]); v->narray[i]=NULL;
+                        }
+                
+                free(v->narray); v->narray=NULL;
+                }
+         
+        v2=v->next;
+        free(v);
+        v=v2;
+        }
+vcdsymroot=vcdsymcurr=NULL;
+                    
+free(sym); sym=NULL;  
+                
+if(slisthier) { free(slisthier); slisthier=NULL; }
+
 return(max_time);
 }
 
@@ -1847,16 +1782,37 @@ if(p<r)
 
 int main(int argc, char **argv)
 {
+int i, dostats=0, doclock=0, dochg=0, dodict=0, linear=0;
+
 if(argc>=3)
         {
-	int dostats =  ((argc>=4)&&(!strcmp(argv[3], "-stats"))) || ((argc>=5)&&(!strcmp(argv[4], "-stats")));
-	int doclock =  ((argc>=4)&&(!strcmp(argv[3], "-clockpack"))) || ((argc>=5)&&(!strcmp(argv[4], "-clockpack")));
+	for(i=3;i<argc;i++)
+		{
+		if(!strcmp(argv[i], "-stats")) dostats = 1;
+		else if(!strcmp(argv[i], "-clockpack")) doclock = 1;
+		else if(!strcmp(argv[i], "-chgpack")) dochg = 1;
+		else if(!strcmp(argv[i], "-linear")) linear = 1;
+		else if(!strcmp(argv[i], "-dictpack")) 
+			{
+			dodict = 1;
+			if((i!=(argc-1))&&(argv[i+1][0]!='-'))
+				{
+				dodict = atoi(argv[++i]);
+				}
+				else
+				{
+				printf("-dictpack missing width parameter, defaulting to 16!\n");
+				dodict = 16;
+				}
+			}
+		}
 
-        vcd_main(argv[1], argv[2], dostats, doclock);
+        vcd_main(argv[1], argv[2], dostats, doclock, dochg, dodict, linear);
         }
         else
         {
-        printf("Usage:\n------\n%s filename.vcd[.gz] filename.lxt [-stats][-clockpack]\nUse \"-\" as a filename to accept uncompressed input from stdin.\n",argv[0]);
+        printf("Usage:\n------\n%s filename.vcd[.gz] filename.lxt [-stats][-clockpack][-chgpack][-linear][-dictpack minwidth]\n"
+		"Use \"-\" as a filename to accept uncompressed input from stdin.\n",argv[0]);
         }
 exit(0);
 }
