@@ -10,6 +10,7 @@
 #include <gtk/gtk.h>
 #include "analyzer.h"
 #include "aet.h"
+#include "lx2.h"
 #include "vcd.h"
 #include "debug.h"
 
@@ -334,6 +335,48 @@ for(i=fetchlow(h_selectedtree)->which;i<=fetchhigh(h_selectedtree)->which;i++)
 		}
         }
 
+/* LX2 */
+if(is_lx2)
+	{
+	int pre_import = 0;
+
+	for(i=fetchlow(h_selectedtree)->which;i<=fetchhigh(h_selectedtree)->which;i++)
+	        {
+	        struct symbol *s, *t;  
+	        s=facs[i];
+		t=s->vec_root;
+		if((t)&&(autocoalesce))
+			{
+			if(t->selected)
+				{
+				while(t)
+					{
+					if(t->n->mvlfac)
+						{
+						lx2_set_fac_process_mask(t->n);
+						pre_import++;
+						}
+					t=t->vec_chain;
+					}
+				}
+			}
+			else
+			{
+			if(s->n->mvlfac)
+				{
+				lx2_set_fac_process_mask(s->n);
+				pre_import++;
+				}
+			}
+	        }
+
+	if(pre_import)
+		{
+		lx2_import_masked();	
+		}
+	}
+/* LX2 */
+
 for(i=fetchlow(h_selectedtree)->which;i<=fetchhigh(h_selectedtree)->which;i++)
         {
 	int len;
@@ -387,6 +430,48 @@ for(i=fetchlow(h_selectedtree)->which;i<=fetchhigh(h_selectedtree)->which;i++)
 		}
         }
 
+/* LX2 */
+if(is_lx2)
+	{
+	int pre_import = 0;
+
+	for(i=fetchlow(h_selectedtree)->which;i<=fetchhigh(h_selectedtree)->which;i++)
+	        {
+	        struct symbol *s, *t;  
+	        s=facs[i];
+		t=s->vec_root;
+		if((t)&&(autocoalesce))
+			{
+			if(t->selected)
+				{
+				while(t)
+					{
+					if(t->n->mvlfac)
+						{
+						lx2_set_fac_process_mask(t->n);
+						pre_import++;
+						}
+					t=t->vec_chain;
+					}
+				}
+			}
+			else
+			{
+			if(s->n->mvlfac)
+				{
+				lx2_set_fac_process_mask(s->n);
+				pre_import++;
+				}
+			}
+	        }
+
+	if(pre_import)
+		{
+		lx2_import_masked();	
+		}
+	}
+/* LX2 */
+
 for(i=fetchlow(h_selectedtree)->which;i<=fetchhigh(h_selectedtree)->which;i++)
         {
 	int len;
@@ -454,6 +539,48 @@ for(i=fetchlow(h_selectedtree)->which;i<=fetchhigh(h_selectedtree)->which;i++)
 		}
         }
 
+/* LX2 */
+if(is_lx2)
+	{
+	int pre_import = 0;
+
+	for(i=fetchlow(h_selectedtree)->which;i<=fetchhigh(h_selectedtree)->which;i++)
+	        {
+	        struct symbol *s, *t;  
+	        s=facs[i];
+		t=s->vec_root;
+		if((t)&&(autocoalesce))
+			{
+			if(t->selected)
+				{
+				while(t)
+					{
+					if(t->n->mvlfac)
+						{
+						lx2_set_fac_process_mask(t->n);
+						pre_import++;
+						}
+					t=t->vec_chain;
+					}
+				}
+			}
+			else
+			{
+			if(s->n->mvlfac)
+				{
+				lx2_set_fac_process_mask(s->n);
+				pre_import++;
+				}
+			}
+	        }
+
+	if(pre_import)
+		{
+		lx2_import_masked();	
+		}
+	}
+/* LX2 */
+
 for(i=fetchlow(h_selectedtree)->which;i<=fetchhigh(h_selectedtree)->which;i++)
         {
 	int len;
