@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 1999-2001.
+ * Copyright (c) Tony Bybell 1999-2003.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -13,8 +13,10 @@
  * window and don't do the op. same for "dnd_state".
  */
 
+#include "gtk12compat.h"
 #include "menu.h"
 #include "vcd.h"
+#include "lx2.h"
 #ifndef _MSC_VER
         #include <unistd.h>
 #endif
@@ -82,6 +84,7 @@ WV_MENU_ALPHA2,
 WV_MENU_LEX,
 WV_MENU_RVS,
 WV_MENU_SPS,
+WV_MENU_SEP7B,
 WV_MENU_SSR,
 WV_MENU_SSH,
 WV_MENU_SST,
@@ -671,6 +674,11 @@ if(helpbox_is_active)
         }
 	else
 	{
+	if(shadow_straces)
+		{
+		delete_strace_context();
+		}
+
 	strace_maketimetrace(0);
   
 	MaxSignalLength();
@@ -922,14 +930,11 @@ if(helpbox_is_active)
 		" When used upon multi-bit vectors that contain "
 		" real valued traces, those traces will expand to their normal \"correct\" values,"
 		" not individual bits."
-                "  Note that this function is disabled when"
-                " Pattern Search is active."
         );
         return;
         }
 
 
-if(straces) { strace_error(); return; } /* don't mess with sigs when strace active */
 if(dnd_state) { dnd_error(); return; } /* don't mess with sigs when dnd active */
 
 DEBUG(printf("Expand Traces\n"));
@@ -1039,7 +1044,6 @@ int tmpi,dirty=0;
 nptr bitblast_parent;
 int bitblast_delta=0;
 
-if(straces) { strace_error(); return; } /* don't mess with sigs when strace active */
 if(dnd_state) { dnd_error(); return; } /* don't mess with sigs when dnd active */
 
 DEBUG(printf("Combine Traces\n"));
@@ -1311,13 +1315,10 @@ if(helpbox_is_active)
                 " be placed in the cut buffer."
 		" It will function seemingly randomly"
 		" when used upon real valued single-bit traces."
-                "  Note that this function is disabled when"
-                " Pattern Search is active."
         );
         return;
         }
 
-if(straces) { strace_error(); return; } /* don't mess with sigs when strace active */
 if(dnd_state) { dnd_error(); return; } /* don't mess with sigs when dnd active */
 menu_combine(1); /* down */
 }
@@ -1335,13 +1336,10 @@ if(helpbox_is_active)
                 " be placed in the cut buffer."
 		" It will function seemingly randomly"
 		" when used upon real valued single-bit traces."
-                "  Note that this function is disabled when"
-                " Pattern Search is active."
         );
         return;
         }
 
-if(straces) { strace_error(); return; } /* don't mess with sigs when strace active */
 if(dnd_state) { dnd_error(); return; } /* don't mess with sigs when dnd active */
 menu_combine(0); /* up */
 }
@@ -1365,14 +1363,11 @@ if(helpbox_is_active)
 		" The resulting bits are converted to traces and inserted after the"
 		" last converted trace with the pre-conversion traces"
 		" being placed in the cut buffer."
-                "  Note that this function is disabled when"
-                " Pattern Search is active."
         );
         return;
         }
 
 
-if(straces) { strace_error(); return; } /* don't mess with sigs when strace active */
 if(dnd_state) { dnd_error(); return; } /* don't mess with sigs when dnd active */
 
 DEBUG(printf("Reduce Singlebit Vex\n"));
@@ -1740,13 +1735,10 @@ if(helpbox_is_active)
                 " Various functions are provided in the Signal Search Tree requester"
                 " which allow searching a treelike hierarchy and bundling"
                 " (coalescing individual bits into a single vector)."
-                "  Note that this function is disabled when"
-                " Pattern Search is active."
         );
         return;
         }
 
-if(straces) { strace_error(); return; } /* don't mess with sigs when strace active */
 if(dnd_state) { dnd_error(); return; } /* don't mess with sigs when dnd active */
 
 treebox("Signal Search Tree",menu_treesearch_cleanup);
@@ -1868,13 +1860,10 @@ if(helpbox_is_active)
                 " Any aliased traces will have their names restored to their"
 		" original names.  As vectors get their names from aliases,"
 		" vector aliases will not be removed."
-                "  Note that this function is disabled when"
-                " Pattern Search is active."
         );
         return;
         }
 
-if(straces) { strace_error(); return; } /* don't mess with sigs when strace active */
 if(dnd_state) { dnd_error(); return; } /* don't mess with sigs when dnd active */
 
 t=traces.first;
@@ -1970,13 +1959,10 @@ if(helpbox_is_active)
 		" will have no such designation.  The purpose of this is to"
 		" provide a fast method of determining which trace names are"
 		" real and which ones are aliases."
-                "  Note that this function is disabled when"
-                " Pattern Search is active."
         );
         return;
         }
 
-if(straces) { strace_error(); return; } /* don't mess with sigs when strace active */
 if(dnd_state) { dnd_error(); return; } /* don't mess with sigs when dnd active */
 
 while(t)
@@ -2015,13 +2001,10 @@ if(helpbox_is_active)
         help_text(
 		" provides an easy means of adding traces to the display in a text based"
 		" treelike fashion."
-                "  Note that this function is disabled when"
-                " Pattern Search is active."
         );
         return;
         }
 
-if(straces) { strace_error(); return; } /* don't mess with sigs when strace active */
 if(dnd_state) { dnd_error(); return; } /* don't mess with sigs when dnd active */
 
 hier_searchbox("Hierarchy Search",menu_hiersearch_cleanup);
@@ -2045,13 +2028,10 @@ if(helpbox_is_active)
 		" Various functions are provided in the Signal Search requester"
 		" which allow searching using POSIX regular expressions and bundling"
 		" (coalescing individual bits into a single vector). "
-                " Note that this function is disabled when"
-                " Pattern Search is active."
         );
         return;
         }
 
-if(straces) { strace_error(); return; } /* don't mess with sigs when strace active */
 if(dnd_state) { dnd_error(); return; } /* don't mess with sigs when dnd active */
 
 searchbox("Signal Search",menu_signalsearch_cleanup);
@@ -2156,6 +2136,7 @@ static void
 menu_write_save_cleanup(GtkWidget *widget, gpointer data)
 {
 FILE *wave;
+struct strace *st;
 
 if(filesel_ok)
 if(!(wave=fopen(*fileselbox_text,"wb")))
@@ -2251,6 +2232,116 @@ if(!(wave=fopen(*fileselbox_text,"wb")))
 			}
 		t=t->next;
 		}
+
+	if(timearray)
+		{
+		if(shadow_straces)
+			{
+			swap_strace_contexts();
+
+			st=straces;
+			if(straces)
+				{
+				fprintf(wave, "!%d%d%d%d%d%d\n", logical_mutex[0], logical_mutex[1], logical_mutex[2], logical_mutex[3], logical_mutex[4], logical_mutex[5]);
+				}
+
+			while(st)
+				{
+				if(st->value==ST_STRING)
+					{
+					fprintf(wave, "?\"%s\n", st->string ? st->string : ""); /* search type for this trace is string.. */
+					}
+					else
+					{
+					fprintf(wave, "?%02x\n", (unsigned char)st->value);	/* else search type for this trace.. */
+					}
+			
+				t=st->trace;
+
+				if(t->flags!=def)
+					{
+					fprintf(wave,"@%x\n",def=t->flags);
+					}
+
+				if((t->shift)||((prevshift)&&(!t->shift)))
+					{
+					fprintf(wave,">"TTFormat"\n", t->shift);
+					}
+				prevshift=t->shift;
+
+				if(!(t->flags&TR_BLANK))	
+					{
+					if(t->vector)
+						{
+						int i;
+						nptr *nodes;
+
+						fprintf(wave,"#%s",t->name);
+
+						nodes=t->n.vec->bits->nodes;
+						for(i=0;i<t->n.vec->nbits;i++)
+							{
+							if(nodes[i]->expansion)
+								{
+								fprintf(wave," (%d)%s",nodes[i]->expansion->parentbit, nodes[i]->expansion->parent->nname);
+								}
+								else
+								{
+								fprintf(wave," %s",nodes[i]->nname);
+								}
+							}
+						fprintf(wave,"\n");
+						}
+						else
+						{
+						if(t->is_alias)
+							{
+							if(t->n.nd->expansion)
+								{
+								fprintf(wave,"+%s (%d)%s\n",t->name+2,t->n.nd->expansion->parentbit, t->n.nd->expansion->parent->nname);
+								}
+								else
+								{
+								fprintf(wave,"+%s %s\n",t->name+2,t->n.nd->nname);
+								}
+							}
+							else
+							{
+							if(t->n.nd->expansion)
+								{
+								fprintf(wave,"(%d)%s\n",t->n.nd->expansion->parentbit, t->n.nd->expansion->parent->nname);
+								}
+								else
+								{
+								fprintf(wave,"%s\n",t->n.nd->nname);
+								}
+							}
+						}
+					}
+
+				st=st->next;
+				} /* while(st)... */
+
+			if(straces)
+				{
+				fprintf(wave, "!!\n");	/* mark end of strace region */
+				}
+		
+				swap_strace_contexts();
+			}
+			else
+			{
+			struct mprintf_buff_t *mt = mprintf_buff_head;
+
+			while(mt)	
+				{
+				fprintf(wave, "%s", mt->str);
+				mt=mt->next;
+				}
+			}
+
+		} /* if(timearray)... */
+
 	fclose(wave);
 	}
 
@@ -2284,7 +2375,9 @@ FILE *wave;
 if(filesel_ok)
 	{
 	char *wname;
+        char *str;
         int wave_is_compressed;
+
 	DEBUG(printf("Read Save Fini: %s\n", *fileselbox_text));
         
         wname=*fileselbox_text;
@@ -2292,7 +2385,6 @@ if(filesel_ok)
         if(((strlen(wname)>2)&&(!strcmp(wname+strlen(wname)-3,".gz")))||
           ((strlen(wname)>3)&&(!strcmp(wname+strlen(wname)-4,".zip"))))
                 {
-                char *str;
                 str=wave_alloca(strlen(wname)+5+1);
                 strcpy(str,"zcat ");
                 strcpy(str+5,wname);
@@ -2315,14 +2407,62 @@ if(filesel_ok)
                 else
                 {
                 char *iline;      
+		char any_shadow = 0;
+
+		if(is_lx2)
+			{
+	                while((iline=fgetmalloc(wave)))
+	                        {
+	                        parsewavline_lx2(iline);
+	                        free_2(iline);
+	                        }
+
+			lx2_import_masked();
+
+			if(wave_is_compressed)
+		                {
+				pclose(wave);
+		                wave=popen(str,"r");
+		                }
+		                else
+		                {   
+				fclose(wave);
+		                wave=fopen(wname,"rb");
+		                }
+
+		        if(!wave)  
+		                {  
+		                fprintf(stderr, "Error opening save file '%s' for reading.\n",*fileselbox_text);
+				perror("Why");
+				errno=0;
+				return;
+		                }
+			}
 
                 default_flags=TR_RJUSTIFY;
 		shift_timebase_default_for_add=LLDescriptor(0);
+
                 while((iline=fgetmalloc(wave)))
                         {
                         parsewavline(iline);
+			any_shadow |= shadow_active;
                         free_2(iline);
                         }
+
+		if(any_shadow)
+			{
+			if(shadow_straces)
+				{
+				shadow_active = 1;
+
+				swap_strace_contexts();
+				strace_maketimetrace(1);
+				swap_strace_contexts();
+
+				shadow_active = 0;
+				}
+			}
+
                 default_flags=TR_RJUSTIFY;
 		shift_timebase_default_for_add=LLDescriptor(0);
 		update_markertime(time_trunc(tims.marker));
@@ -2368,13 +2508,10 @@ if(helpbox_is_active)
                 " inserts a blank trace after the last highlighted trace."
                 " If no traces are highlighted, the blank is inserted after"
 		" the last trace."
-		"  Note that this function is disabled when"
-		" Pattern Search is active."
         );
         return;
         }
 
-if(straces) { strace_error(); return; } /* don't mess with sigs when strace active */
 if(dnd_state) { dnd_error(); return; } /* don't mess with sigs when dnd active */
 
 DEBUG(printf("Insert Blank Trace\n"));
@@ -2405,13 +2542,10 @@ if(helpbox_is_active)
                 " inserts a comment trace after the last highlighted trace."
                 " If no traces are highlighted, the comment is inserted after"
 		" the last trace."
-		"  Note that this function is disabled when"
-		" Pattern Search is active."
         );
         return;
         }
 
-if(straces) { strace_error(); return; } /* don't mess with sigs when strace active */
 if(dnd_state) { dnd_error(); return; } /* don't mess with sigs when dnd active */
 
 DEBUG(printf("Insert Comment Trace\n"));
@@ -3048,13 +3182,10 @@ if(helpbox_is_active)
                 " removes highlighted signals from the display and places them" 
 		" in an offscreen cut buffer for later Paste operations. "
 		" Cut implicitly destroys the previous contents of the cut buffer."
-                "  Note that this function is disabled when"
-                " Pattern Search is active."
         );
         return;
         }                
 
-if(straces) { strace_error(); return; } /* don't mess with sigs when strace active */
 if(dnd_state) { dnd_error(); return; } /* don't mess with sigs when dnd active */
 
 DEBUG(printf("Cut Traces\n"));
@@ -3083,14 +3214,11 @@ if(helpbox_is_active)
 		" the last highlighted signal, or at the end of the display"
 		" if no signal is highlighted."
                 " Paste implicitly destroys the previous contents of the cut buffer."
-                "  Note that this function is disabled when"
-                " Pattern Search is active."
         );
         return;
         }
 
 
-if(straces) { strace_error(); return; } /* don't mess with sigs when strace active */
 if(dnd_state) { dnd_error(); return; } /* don't mess with sigs when dnd active */
 
 DEBUG(printf("Paste Traces\n"));
@@ -3196,126 +3324,127 @@ GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(item_factory,
 static GtkItemFactoryEntry menu_items[] =
 {
 #ifndef _MSC_VER
-    "/File/Open New Viewer", "Pause", menu_new_viewer, WV_MENU_FONV, "<Item>",
-    "/File/<separator>", NULL, NULL, WV_MENU_SEP0, "<Separator>",
+    WAVE_GTKIFE("/File/Open New Viewer", "Pause", menu_new_viewer, WV_MENU_FONV, "<Item>"),
+    WAVE_GTKIFE("/File/<separator>", NULL, NULL, WV_MENU_SEP0, "<Separator>"),
 #endif
-    "/File/Print To File", "Print", menu_print, WV_MENU_FPTF, "<Item>",
-    "/File/<separator>", NULL, NULL, WV_MENU_SEP1, "<Separator>",
-    "/File/Read Save File", "<Control>R", menu_read_save_file, WV_MENU_FRSF, "<Item>",
-    "/File/Write Save File", "<Control>W", menu_write_save_file, WV_MENU_FWSF, "<Item>",
-    "/File/<separator>", NULL, NULL, WV_MENU_SEP2, "<Separator>",
-    "/File/Quit/Yes, Quit", "<Alt>Q", menu_quit, WV_MENU_FQY, "<Item>",
-    "/File/Quit/Don't Quit", NULL, NULL, WV_MENU_FQN, "<Item>",
+    WAVE_GTKIFE("/File/Print To File", "Print", menu_print, WV_MENU_FPTF, "<Item>"),
+    WAVE_GTKIFE("/File/<separator>", NULL, NULL, WV_MENU_SEP1, "<Separator>"),
+    WAVE_GTKIFE("/File/Read Save File", "<Control>R", menu_read_save_file, WV_MENU_FRSF, "<Item>"),
+    WAVE_GTKIFE("/File/Write Save File", "<Control>W", menu_write_save_file, WV_MENU_FWSF, "<Item>"),
+    WAVE_GTKIFE("/File/<separator>", NULL, NULL, WV_MENU_SEP2, "<Separator>"),
+    WAVE_GTKIFE("/File/Quit/Yes, Quit", "<Alt>Q", menu_quit, WV_MENU_FQY, "<Item>"),
+    WAVE_GTKIFE("/File/Quit/Don't Quit", NULL, NULL, WV_MENU_FQN, "<Item>"),
 
-    "/Edit/Set Trace Max Hier", "<Control>T", menu_set_max_hier, WV_MENU_ESTMH, "<Item>",
-    "/Edit/<separator>", NULL, NULL, WV_MENU_SEP3, "<Separator>",
-    "/Edit/Insert Blank", "<Control>B", menu_insert_blank_traces, WV_MENU_EIB, "<Item>",
-    "/Edit/Insert Comment", "<Control>C", menu_insert_comment_traces, WV_MENU_EIC, "<Item>",
-    "/Edit/Alias Highlighted Trace", "<Alt>A", menu_alias, WV_MENU_EAHT, "<Item>",
-    "/Edit/Remove Highlighted Aliases", "<Shift><Alt>A", menu_remove_aliases, WV_MENU_ERHA, "<Item>",
-    "/Edit/Cut", "<Alt>C", menu_cut_traces, WV_MENU_EC, "<Item>",
-    "/Edit/Paste", "<Alt>P", menu_paste_traces, WV_MENU_EP, "<Item>",
-    "/Edit/<separator>", NULL, NULL, WV_MENU_SEP4, "<Separator>",
-    "/Edit/Expand", "F3", menu_expand, WV_MENU_EE, "<Item>",
-    "/Edit/Combine Down", "F4", menu_combine_down, WV_MENU_ECD, "<Item>",
-    "/Edit/Combine Up", "F5", menu_combine_up, WV_MENU_ECU, "<Item>",
-    "/Edit/Reduce Single Bit Vectors", "F6", menu_reduce_singlebit_vex, WV_MENU_ERSBV, "<Item>",
-    "/Edit/<separator>", NULL, NULL, WV_MENU_SEP5, "<Separator>",
-    "/Edit/Data Format/Hex", "<Alt>X", menu_dataformat_hex, WV_MENU_EDFH, "<Item>",
-    "/Edit/Data Format/Decimal", "<Alt>D", menu_dataformat_dec, WV_MENU_EDFD, "<Item>",
-    "/Edit/Data Format/Signed Decimal", NULL, menu_dataformat_signed, WV_MENU_EDFSD, "<Item>",
-    "/Edit/Data Format/Binary", "<Alt>B", menu_dataformat_bin, WV_MENU_EDFB, "<Item>",
-    "/Edit/Data Format/Octal", "<Alt>O", menu_dataformat_oct, WV_MENU_EDFO, "<Item>",
-    "/Edit/Data Format/ASCII", NULL, menu_dataformat_ascii, WV_MENU_EDFA, "<Item>",
-    "/Edit/Data Format/Right Justify/On", "<Alt>J", menu_dataformat_rjustify_on, WV_MENU_EDFRJON, "<Item>",
-    "/Edit/Data Format/Right Justify/Off", "<Shift><Alt>J", menu_dataformat_rjustify_off, WV_MENU_EDFRJOFF, "<Item>",
-    "/Edit/Data Format/Invert/On", "<Alt>I", menu_dataformat_invert_on, WV_MENU_EDFION, "<Item>",
-    "/Edit/Data Format/Invert/Off", "<Shift><Alt>I", menu_dataformat_invert_off, WV_MENU_EDFIOFF, "<Item>",
-    "/Edit/Data Format/Reverse Bits/On", "<Alt>V", menu_dataformat_reverse_on, WV_MENU_EDFRON, "<Item>",
-    "/Edit/Data Format/Reverse Bits/Off", "<Shift><Alt>V", menu_dataformat_reverse_off, WV_MENU_EDFROFF, "<Item>",
-    "/Edit/Show-Change All Highlighted", "<Control>S", menu_showchangeall, WV_MENU_ESCAH, "<Item>",
-    "/Edit/Show-Change First Highlighted", "<Control>F", menu_showchange, WV_MENU_ESCFH, "<Item>",
-    "/Edit/<separator>", NULL, NULL, WV_MENU_SEP6, "<Separator>",
-    "/Edit/Time Warp/Warp Marked", NULL, menu_warp_traces, WV_MENU_WARP, "<Item>",
-    "/Edit/Time Warp/Unwarp Marked", NULL, menu_unwarp_traces, WV_MENU_UNWARP, "<Item>",
-    "/Edit/Time Warp/Unwarp All", NULL, menu_unwarp_traces_all, WV_MENU_UNWARPA, "<Item>",
-    "/Edit/<separator>", NULL, NULL, WV_MENU_SEP7A, "<Separator>",
-    "/Edit/Exclude", "<Shift><Alt>E", menu_dataformat_exclude_on, WV_MENU_EEX, "<Item>",
-    "/Edit/Show", "<Shift><Alt>S", menu_dataformat_exclude_off, WV_MENU_ESH, "<Item>",
-    "/Edit/<separator>", NULL, NULL, WV_MENU_SEP6A, "<Separator>",
-    "/Edit/Highlight Regexp", "<Alt>R", menu_regexp_highlight, WV_MENU_EHR, "<Item>",
-    "/Edit/UnHighlight Regexp", "<Shift><Alt>R", menu_regexp_unhighlight, WV_MENU_EUHR, "<Item>",
-    "/Edit/Highlight All", "<Alt>H", menu_dataformat_highlight_all, WV_MENU_EHA, "<Item>",
-    "/Edit/UnHighlight All", "<Shift><Alt>H", menu_dataformat_unhighlight_all, WV_MENU_EUHA, "<Item>",
-    "/Edit/<separator>", NULL, NULL, WV_MENU_SEP6B, "<Separator>",
-    "/Edit/Sort/Alphabetize All", NULL, menu_alphabetize, WV_MENU_ALPHA, "<Item>",
-    "/Edit/Sort/Alphabetize All (CaseIns)", NULL, menu_alphabetize2, WV_MENU_ALPHA2, "<Item>",
-    "/Edit/Sort/Sigsort All", NULL, menu_lexize, WV_MENU_LEX, "<Item>",
-    "/Edit/Sort/Reverse All", NULL, menu_reverse, WV_MENU_RVS, "<Item>",
+    WAVE_GTKIFE("/Edit/Set Trace Max Hier", "<Control>T", menu_set_max_hier, WV_MENU_ESTMH, "<Item>"),
+    WAVE_GTKIFE("/Edit/<separator>", NULL, NULL, WV_MENU_SEP3, "<Separator>"),
+    WAVE_GTKIFE("/Edit/Insert Blank", "<Control>B", menu_insert_blank_traces, WV_MENU_EIB, "<Item>"),
+    WAVE_GTKIFE("/Edit/Insert Comment", "<Control>C", menu_insert_comment_traces, WV_MENU_EIC, "<Item>"),
+    WAVE_GTKIFE("/Edit/Alias Highlighted Trace", "<Alt>A", menu_alias, WV_MENU_EAHT, "<Item>"),
+    WAVE_GTKIFE("/Edit/Remove Highlighted Aliases", "<Shift><Alt>A", menu_remove_aliases, WV_MENU_ERHA, "<Item>"),
+    WAVE_GTKIFE("/Edit/Cut", "<Alt>C", menu_cut_traces, WV_MENU_EC, "<Item>"),
+    WAVE_GTKIFE("/Edit/Paste", "<Alt>P", menu_paste_traces, WV_MENU_EP, "<Item>"),
+    WAVE_GTKIFE("/Edit/<separator>", NULL, NULL, WV_MENU_SEP4, "<Separator>"),
+    WAVE_GTKIFE("/Edit/Expand", "F3", menu_expand, WV_MENU_EE, "<Item>"),
+    WAVE_GTKIFE("/Edit/Combine Down", "F4", menu_combine_down, WV_MENU_ECD, "<Item>"),
+    WAVE_GTKIFE("/Edit/Combine Up", "F5", menu_combine_up, WV_MENU_ECU, "<Item>"),
+    WAVE_GTKIFE("/Edit/Reduce Single Bit Vectors", "F6", menu_reduce_singlebit_vex, WV_MENU_ERSBV, "<Item>"),
+    WAVE_GTKIFE("/Edit/<separator>", NULL, NULL, WV_MENU_SEP5, "<Separator>"),
+    WAVE_GTKIFE("/Edit/Data Format/Hex", "<Alt>X", menu_dataformat_hex, WV_MENU_EDFH, "<Item>"),
+    WAVE_GTKIFE("/Edit/Data Format/Decimal", "<Alt>D", menu_dataformat_dec, WV_MENU_EDFD, "<Item>"),
+    WAVE_GTKIFE("/Edit/Data Format/Signed Decimal", NULL, menu_dataformat_signed, WV_MENU_EDFSD, "<Item>"),
+    WAVE_GTKIFE("/Edit/Data Format/Binary", "<Alt>B", menu_dataformat_bin, WV_MENU_EDFB, "<Item>"),
+    WAVE_GTKIFE("/Edit/Data Format/Octal", "<Alt>O", menu_dataformat_oct, WV_MENU_EDFO, "<Item>"),
+    WAVE_GTKIFE("/Edit/Data Format/ASCII", NULL, menu_dataformat_ascii, WV_MENU_EDFA, "<Item>"),
+    WAVE_GTKIFE("/Edit/Data Format/Right Justify/On", "<Alt>J", menu_dataformat_rjustify_on, WV_MENU_EDFRJON, "<Item>"),
+    WAVE_GTKIFE("/Edit/Data Format/Right Justify/Off", "<Shift><Alt>J", menu_dataformat_rjustify_off, WV_MENU_EDFRJOFF, "<Item>"),
+    WAVE_GTKIFE("/Edit/Data Format/Invert/On", "<Alt>I", menu_dataformat_invert_on, WV_MENU_EDFION, "<Item>"),
+    WAVE_GTKIFE("/Edit/Data Format/Invert/Off", "<Shift><Alt>I", menu_dataformat_invert_off, WV_MENU_EDFIOFF, "<Item>"),
+    WAVE_GTKIFE("/Edit/Data Format/Reverse Bits/On", "<Alt>V", menu_dataformat_reverse_on, WV_MENU_EDFRON, "<Item>"),
+    WAVE_GTKIFE("/Edit/Data Format/Reverse Bits/Off", "<Shift><Alt>V", menu_dataformat_reverse_off, WV_MENU_EDFROFF, "<Item>"),
+    WAVE_GTKIFE("/Edit/Show-Change All Highlighted", "<Control>S", menu_showchangeall, WV_MENU_ESCAH, "<Item>"),
+    WAVE_GTKIFE("/Edit/Show-Change First Highlighted", "<Control>F", menu_showchange, WV_MENU_ESCFH, "<Item>"),
+    WAVE_GTKIFE("/Edit/<separator>", NULL, NULL, WV_MENU_SEP6, "<Separator>"),
+    WAVE_GTKIFE("/Edit/Time Warp/Warp Marked", NULL, menu_warp_traces, WV_MENU_WARP, "<Item>"),
+    WAVE_GTKIFE("/Edit/Time Warp/Unwarp Marked", NULL, menu_unwarp_traces, WV_MENU_UNWARP, "<Item>"),
+    WAVE_GTKIFE("/Edit/Time Warp/Unwarp All", NULL, menu_unwarp_traces_all, WV_MENU_UNWARPA, "<Item>"),
+    WAVE_GTKIFE("/Edit/<separator>", NULL, NULL, WV_MENU_SEP7A, "<Separator>"),
+    WAVE_GTKIFE("/Edit/Exclude", "<Shift><Alt>E", menu_dataformat_exclude_on, WV_MENU_EEX, "<Item>"),
+    WAVE_GTKIFE("/Edit/Show", "<Shift><Alt>S", menu_dataformat_exclude_off, WV_MENU_ESH, "<Item>"),
+    WAVE_GTKIFE("/Edit/<separator>", NULL, NULL, WV_MENU_SEP6A, "<Separator>"),
+    WAVE_GTKIFE("/Edit/Highlight Regexp", "<Alt>R", menu_regexp_highlight, WV_MENU_EHR, "<Item>"),
+    WAVE_GTKIFE("/Edit/UnHighlight Regexp", "<Shift><Alt>R", menu_regexp_unhighlight, WV_MENU_EUHR, "<Item>"),
+    WAVE_GTKIFE("/Edit/Highlight All", "<Alt>H", menu_dataformat_highlight_all, WV_MENU_EHA, "<Item>"),
+    WAVE_GTKIFE("/Edit/UnHighlight All", "<Shift><Alt>H", menu_dataformat_unhighlight_all, WV_MENU_EUHA, "<Item>"),
+    WAVE_GTKIFE("/Edit/<separator>", NULL, NULL, WV_MENU_SEP6B, "<Separator>"),
+    WAVE_GTKIFE("/Edit/Sort/Alphabetize All", NULL, menu_alphabetize, WV_MENU_ALPHA, "<Item>"),
+    WAVE_GTKIFE("/Edit/Sort/Alphabetize All (CaseIns)", NULL, menu_alphabetize2, WV_MENU_ALPHA2, "<Item>"),
+    WAVE_GTKIFE("/Edit/Sort/Sigsort All", NULL, menu_lexize, WV_MENU_LEX, "<Item>"),
+    WAVE_GTKIFE("/Edit/Sort/Reverse All", NULL, menu_reverse, WV_MENU_RVS, "<Item>"),
 
-    "/Search/Pattern Search", "<Control>P", menu_tracesearchbox, WV_MENU_SPS, "<Item>",
-    "/Search/Signal Search Regexp", "<Alt>S", menu_signalsearch, WV_MENU_SSR, "<Item>",
-    "/Search/Signal Search Hierarchy", "<Alt>T", menu_hiersearch, WV_MENU_SSH, "<Item>",
-    "/Search/Signal Search Tree", "<Shift><Alt>T", menu_treesearch, WV_MENU_SST, "<Item>",
-    "/Search/<separator>", NULL, NULL, WV_MENU_SEP7, "<Separator>",
-    "/Search/Autocoalesce", NULL, menu_autocoalesce, WV_MENU_ACOL, "<ToggleItem>",
-    "/Search/Autocoalesce Reversal", NULL, menu_autocoalesce_reversal, WV_MENU_ACOLR, "<ToggleItem>",
-    "/Search/Autoname Bundles", NULL, menu_autoname_bundles_on, WV_MENU_ABON, "<ToggleItem>",
-    "/Search/Search Hierarchy Grouping", NULL, menu_hgrouping, WV_MENU_HTGP, "<ToggleItem>",
+    WAVE_GTKIFE("/Search/Pattern Search", "<Control>P", menu_tracesearchbox, WV_MENU_SPS, "<Item>"),
+    WAVE_GTKIFE("/Search/<separator>", NULL, NULL, WV_MENU_SEP7B, "<Separator>"),
+    WAVE_GTKIFE("/Search/Signal Search Regexp", "<Alt>S", menu_signalsearch, WV_MENU_SSR, "<Item>"),
+    WAVE_GTKIFE("/Search/Signal Search Hierarchy", "<Alt>T", menu_hiersearch, WV_MENU_SSH, "<Item>"),
+    WAVE_GTKIFE("/Search/Signal Search Tree", "<Shift><Alt>T", menu_treesearch, WV_MENU_SST, "<Item>"),
+    WAVE_GTKIFE("/Search/<separator>", NULL, NULL, WV_MENU_SEP7, "<Separator>"),
+    WAVE_GTKIFE("/Search/Autocoalesce", NULL, menu_autocoalesce, WV_MENU_ACOL, "<ToggleItem>"),
+    WAVE_GTKIFE("/Search/Autocoalesce Reversal", NULL, menu_autocoalesce_reversal, WV_MENU_ACOLR, "<ToggleItem>"),
+    WAVE_GTKIFE("/Search/Autoname Bundles", NULL, menu_autoname_bundles_on, WV_MENU_ABON, "<ToggleItem>"),
+    WAVE_GTKIFE("/Search/Search Hierarchy Grouping", NULL, menu_hgrouping, WV_MENU_HTGP, "<ToggleItem>"),
 
-    "/Time/Move To Time", "F1", menu_movetotime, WV_MENU_TMTT, "<Item>",
-    "/Time/Zoom/Zoom Amount", "F2", menu_zoomsize, WV_MENU_TZZA, "<Item>",
-    "/Time/Zoom/Zoom Base", "<Shift>F2", menu_zoombase, WV_MENU_TZZB, "<Item>",
-    "/Time/Zoom/Zoom In", "<Alt>Z", service_zoom_in, WV_MENU_TZZI, "<Item>",
-    "/Time/Zoom/Zoom Out", "<Shift><Alt>Z", service_zoom_out, WV_MENU_TZZO, "<Item>",
-    "/Time/Zoom/Zoom Best Fit", "<Alt>F", service_zoom_fit, WV_MENU_TZZBF, "<Item>",
-    "/Time/Zoom/Zoom To Start", "Home", service_zoom_left, WV_MENU_TZZTS, "<Item>",
-    "/Time/Zoom/Zoom To End", "End", service_zoom_right, WV_MENU_TZZTE, "<Item>",
-    "/Time/Zoom/Undo Zoom", "<Alt>U", service_zoom_undo, WV_MENU_TZUZ, "<Item>",
-    "/Time/Fetch/Fetch Size", "F7", menu_fetchsize, WV_MENU_TFFS, "<Item>",
-    "/Time/Fetch/Fetch ->", "<Alt>2", fetch_right, WV_MENU_TFFR, "<Item>",
-    "/Time/Fetch/Fetch <-", "<Alt>1", fetch_left, WV_MENU_TFFL, "<Item>",
-    "/Time/Discard/Discard ->", "<Alt>4", discard_right, WV_MENU_TDDR, "<Item>",
-    "/Time/Discard/Discard <-", "<Alt>3", discard_left, WV_MENU_TDDL, "<Item>",
-    "/Time/Shift/Shift ->", "<Alt>6", service_right_shift, WV_MENU_TSSR, "<Item>",
-    "/Time/Shift/Shift <-", "<Alt>5", service_left_shift, WV_MENU_TSSL, "<Item>",
-    "/Time/Page/Page ->", "<Alt>8", service_right_page, WV_MENU_TPPR, "<Item>",
-    "/Time/Page/Page <-", "<Alt>7", service_left_page, WV_MENU_TPPL, "<Item>",
+    WAVE_GTKIFE("/Time/Move To Time", "F1", menu_movetotime, WV_MENU_TMTT, "<Item>"),
+    WAVE_GTKIFE("/Time/Zoom/Zoom Amount", "F2", menu_zoomsize, WV_MENU_TZZA, "<Item>"),
+    WAVE_GTKIFE("/Time/Zoom/Zoom Base", "<Shift>F2", menu_zoombase, WV_MENU_TZZB, "<Item>"),
+    WAVE_GTKIFE("/Time/Zoom/Zoom In", "<Alt>Z", service_zoom_in, WV_MENU_TZZI, "<Item>"),
+    WAVE_GTKIFE("/Time/Zoom/Zoom Out", "<Shift><Alt>Z", service_zoom_out, WV_MENU_TZZO, "<Item>"),
+    WAVE_GTKIFE("/Time/Zoom/Zoom Best Fit", "<Alt>F", service_zoom_fit, WV_MENU_TZZBF, "<Item>"),
+    WAVE_GTKIFE("/Time/Zoom/Zoom To Start", "Home", service_zoom_left, WV_MENU_TZZTS, "<Item>"),
+    WAVE_GTKIFE("/Time/Zoom/Zoom To End", "End", service_zoom_right, WV_MENU_TZZTE, "<Item>"),
+    WAVE_GTKIFE("/Time/Zoom/Undo Zoom", "<Alt>U", service_zoom_undo, WV_MENU_TZUZ, "<Item>"),
+    WAVE_GTKIFE("/Time/Fetch/Fetch Size", "F7", menu_fetchsize, WV_MENU_TFFS, "<Item>"),
+    WAVE_GTKIFE("/Time/Fetch/Fetch ->", "<Alt>2", fetch_right, WV_MENU_TFFR, "<Item>"),
+    WAVE_GTKIFE("/Time/Fetch/Fetch <-", "<Alt>1", fetch_left, WV_MENU_TFFL, "<Item>"),
+    WAVE_GTKIFE("/Time/Discard/Discard ->", "<Alt>4", discard_right, WV_MENU_TDDR, "<Item>"),
+    WAVE_GTKIFE("/Time/Discard/Discard <-", "<Alt>3", discard_left, WV_MENU_TDDL, "<Item>"),
+    WAVE_GTKIFE("/Time/Shift/Shift ->", "<Alt>6", service_right_shift, WV_MENU_TSSR, "<Item>"),
+    WAVE_GTKIFE("/Time/Shift/Shift <-", "<Alt>5", service_left_shift, WV_MENU_TSSL, "<Item>"),
+    WAVE_GTKIFE("/Time/Page/Page ->", "<Alt>8", service_right_page, WV_MENU_TPPR, "<Item>"),
+    WAVE_GTKIFE("/Time/Page/Page <-", "<Alt>7", service_left_page, WV_MENU_TPPL, "<Item>"),
 
-    "/Markers/Show-Change Marker Data", "<Alt>M", menu_markerbox, WV_MENU_MSCMD, "<Item>",
-    "/Markers/Drop Named Marker", "<Alt>N", drop_named_marker, WV_MENU_MDNM, "<Item>",
-    "/Markers/Collect Named Marker", "<Shift><Alt>N", collect_named_marker, WV_MENU_MCNM, "<Item>",
-    "/Markers/Collect All Named Markers", "<Shift><Control><Alt>N", collect_all_named_markers, WV_MENU_MCANM, "<Item>",
-    "/Markers/Delete Primary Marker", "<Shift><Alt>M", delete_unnamed_marker, WV_MENU_MDPM, "<Item>",
-    "/Markers/<separator>", NULL, NULL, WV_MENU_SEP8, "<Separator>",
-    "/Markers/Wave Scrolling", "F9", wave_scrolling_on, WV_MENU_MWSON, "<ToggleItem>",
+    WAVE_GTKIFE("/Markers/Show-Change Marker Data", "<Alt>M", menu_markerbox, WV_MENU_MSCMD, "<Item>"),
+    WAVE_GTKIFE("/Markers/Drop Named Marker", "<Alt>N", drop_named_marker, WV_MENU_MDNM, "<Item>"),
+    WAVE_GTKIFE("/Markers/Collect Named Marker", "<Shift><Alt>N", collect_named_marker, WV_MENU_MCNM, "<Item>"),
+    WAVE_GTKIFE("/Markers/Collect All Named Markers", "<Shift><Control><Alt>N", collect_all_named_markers, WV_MENU_MCANM, "<Item>"),
+    WAVE_GTKIFE("/Markers/Delete Primary Marker", "<Shift><Alt>M", delete_unnamed_marker, WV_MENU_MDPM, "<Item>"),
+    WAVE_GTKIFE("/Markers/<separator>", NULL, NULL, WV_MENU_SEP8, "<Separator>"),
+    WAVE_GTKIFE("/Markers/Wave Scrolling", "F9", wave_scrolling_on, WV_MENU_MWSON, "<ToggleItem>"),
 
-    "/View/Show Grid", "<Alt>G", menu_show_grid, WV_MENU_VSG, "<ToggleItem>",
-    "/View/<separator>", NULL, NULL, WV_MENU_SEP9, "<Separator>",
-    "/View/Show Base Symbols", "<Alt>F1", menu_show_base, WV_MENU_VSBS, "<ToggleItem>",
-    "/View/<separator>", NULL, NULL, WV_MENU_SEP10, "<Separator>",
-    "/View/Dynamic Resize", "<Alt>9", menu_enable_dynamic_resize, WV_MENU_VDR, "<ToggleItem>",
-    "/View/<separator>", NULL, NULL, WV_MENU_SEP11, "<Separator>",
-    "/View/Center Zooms", "F8", menu_center_zooms, WV_MENU_VCZ, "<ToggleItem>",
-    "/View/<separator>", NULL, NULL, WV_MENU_SEP12, "<Separator>",
-    "/View/Toggle Max-Marker", "F10", menu_toggle_max_or_marker, WV_MENU_VTMM, "<Item>",
-    "/View/<separator>", NULL, NULL, WV_MENU_SEP13, "<Separator>",
-    "/View/Constant Marker Update", "F11", menu_enable_constant_marker_update, WV_MENU_VCMU, "<ToggleItem>",
-    "/View/<separator>", NULL, NULL, WV_MENU_SEP14, "<Separator>",
-    "/View/Draw Roundcapped Vectors", "<Alt>F2", menu_use_roundcaps, WV_MENU_VDRV, "<ToggleItem>",
-    "/View/<separator>", NULL, NULL, WV_MENU_SEP15, "<Separator>",
-    "/View/Left Justified Signals", "<Shift>Home", menu_left_justify, WV_MENU_VLJS, "<Item>",
-    "/View/Right Justified Signals", "<Shift>End", menu_right_justify, WV_MENU_VRJS, "<Item>",
-    "/View/<separator>", NULL, NULL, WV_MENU_SEP16, "<Separator>",
-    "/View/Zoom Pow10 Snap", "<Shift>Pause", menu_zoom10_snap, WV_MENU_VZPS, "<ToggleItem>",
-    "/View/Full Precision", "<Alt>Pause", menu_use_full_precision, WV_MENU_VFTP, "<ToggleItem>",
-    "/View/<separator>", NULL, NULL, WV_MENU_SEP17, "<Separator>",
-    "/View/Remove Pattern Marks", NULL, menu_remove_marked, WV_MENU_RMRKS, "<Item>",
-    "/View/<separator>", NULL, NULL, WV_MENU_SEP18, "<Separator>",
-    "/View/LXT Clock Compress to Z", NULL, menu_lxt_clk_compress, WV_MENU_LXTCC2Z, "<ToggleItem>",
+    WAVE_GTKIFE("/View/Show Grid", "<Alt>G", menu_show_grid, WV_MENU_VSG, "<ToggleItem>"),
+    WAVE_GTKIFE("/View/<separator>", NULL, NULL, WV_MENU_SEP9, "<Separator>"),
+    WAVE_GTKIFE("/View/Show Base Symbols", "<Alt>F1", menu_show_base, WV_MENU_VSBS, "<ToggleItem>"),
+    WAVE_GTKIFE("/View/<separator>", NULL, NULL, WV_MENU_SEP10, "<Separator>"),
+    WAVE_GTKIFE("/View/Dynamic Resize", "<Alt>9", menu_enable_dynamic_resize, WV_MENU_VDR, "<ToggleItem>"),
+    WAVE_GTKIFE("/View/<separator>", NULL, NULL, WV_MENU_SEP11, "<Separator>"),
+    WAVE_GTKIFE("/View/Center Zooms", "F8", menu_center_zooms, WV_MENU_VCZ, "<ToggleItem>"),
+    WAVE_GTKIFE("/View/<separator>", NULL, NULL, WV_MENU_SEP12, "<Separator>"),
+    WAVE_GTKIFE("/View/Toggle Max-Marker", "F10", menu_toggle_max_or_marker, WV_MENU_VTMM, "<Item>"),
+    WAVE_GTKIFE("/View/<separator>", NULL, NULL, WV_MENU_SEP13, "<Separator>"),
+    WAVE_GTKIFE("/View/Constant Marker Update", "F11", menu_enable_constant_marker_update, WV_MENU_VCMU, "<ToggleItem>"),
+    WAVE_GTKIFE("/View/<separator>", NULL, NULL, WV_MENU_SEP14, "<Separator>"),
+    WAVE_GTKIFE("/View/Draw Roundcapped Vectors", "<Alt>F2", menu_use_roundcaps, WV_MENU_VDRV, "<ToggleItem>"),
+    WAVE_GTKIFE("/View/<separator>", NULL, NULL, WV_MENU_SEP15, "<Separator>"),
+    WAVE_GTKIFE("/View/Left Justified Signals", "<Shift>Home", menu_left_justify, WV_MENU_VLJS, "<Item>"),
+    WAVE_GTKIFE("/View/Right Justified Signals", "<Shift>End", menu_right_justify, WV_MENU_VRJS, "<Item>"),
+    WAVE_GTKIFE("/View/<separator>", NULL, NULL, WV_MENU_SEP16, "<Separator>"),
+    WAVE_GTKIFE("/View/Zoom Pow10 Snap", "<Shift>Pause", menu_zoom10_snap, WV_MENU_VZPS, "<ToggleItem>"),
+    WAVE_GTKIFE("/View/Full Precision", "<Alt>Pause", menu_use_full_precision, WV_MENU_VFTP, "<ToggleItem>"),
+    WAVE_GTKIFE("/View/<separator>", NULL, NULL, WV_MENU_SEP17, "<Separator>"),
+    WAVE_GTKIFE("/View/Remove Pattern Marks", NULL, menu_remove_marked, WV_MENU_RMRKS, "<Item>"),
+    WAVE_GTKIFE("/View/<separator>", NULL, NULL, WV_MENU_SEP18, "<Separator>"),
+    WAVE_GTKIFE("/View/LXT Clock Compress to Z", NULL, menu_lxt_clk_compress, WV_MENU_LXTCC2Z, "<ToggleItem>"),
 
-    "/Help/WAVE Help", "<Control>H", menu_help, WV_MENU_HWH, "<Item>",
-    "/Help/Wave Version", "<Control>V", menu_version, WV_MENU_HWV, "<Item>",
+    WAVE_GTKIFE("/Help/WAVE Help", "<Control>H", menu_help, WV_MENU_HWH, "<Item>"),
+    WAVE_GTKIFE("/Help/Wave Version", "<Control>V", menu_version, WV_MENU_HWV, "<Item>"),
 };
 
 
