@@ -800,6 +800,46 @@ pr_renderhash(pr_context * prc, int x, TimeType tim)
 }
 
 
+static void 
+pr_renderblackout(pr_context * prc)
+{
+gfloat pageinc;
+TimeType lhs, rhs, lclip, rclip;
+struct blackout_region_t *bt = blackout_regions;
+                         
+if(bt)
+        {
+        pageinc=(gfloat)(((gdouble)wavewidth)*nspx);
+        lhs = tims.start;
+        rhs = pageinc + lhs;
+        
+        while(bt)
+                {
+                if( ((bt->bstart <= lhs) && (bt->bend >= lhs)) || ((bt->bstart >= lhs) && (bt->bstart <= rhs)) )
+                        {
+                        lclip = bt->bstart; rclip = bt->bend;
+                
+                        if(lclip < lhs) lclip = lhs;
+                                else if (lclip > rhs) lclip = rhs;
+
+                        if(rclip < lhs) rclip = lhs;
+                 
+                        lclip -= lhs;
+                        rclip -= lhs;
+                        if(rclip>(wavewidth+1)) rclip = wavewidth+1;
+
+			pr_setgray(prc, 0.80);
+                        pr_draw_box(prc,
+                                (((gdouble)lclip)*pxns), fontheight,
+                                (((gdouble)(rclip))*pxns), waveheight-fontheight);
+                        }
+                 
+                bt=bt->next;
+                }
+        }
+}
+
+
 static void
 pr_rendertimes(pr_context * prc)
 {
@@ -807,6 +847,8 @@ pr_rendertimes(pr_context * prc)
 	int             x, len;
 	char            timebuff[32];
 
+	pr_renderblackout(prc);
+
 	tim = tims.start;
 	tims.end = tims.start + (((gdouble) wavewidth) * nspx);
 
@@ -1164,7 +1206,7 @@ pr_draw_hptr_trace_vector(pr_context * prc, Trptr t, hptr h, int which)
 			x1 = wavewidth;
 
 		/* draw trans */
-		type = vtype(t, h->v.vector);
+		type = (!(h->flags&(HIST_REAL|HIST_STRING))) ? vtype(t,h->v.vector) : 0;
 		if (x0 > -1)
 			if (use_roundcaps) {
 				if (type == 2) {
