@@ -97,6 +97,15 @@ convert_to_reals=atoi_64(str)?1:0;
 return(0);
 }
 
+static int f_cursor_snap(char *str)
+{
+int val;
+DEBUG(printf("f_cursor_snap(\"%s\")\n",str));
+val=atoi_64(str);
+cursor_snap=(val<=0)?0:val;
+return(0);
+}
+
 static int f_disable_tooltips(char *str)
 {
 DEBUG(printf("f_disable_tooltips(\"%s\")\n",str));
@@ -209,6 +218,15 @@ initial_window_x=(val<=0)?-1:val;
 return(0);
 }
 
+static int f_initial_window_xpos(char *str)
+{
+int val;
+DEBUG(printf("f_initial_window_xpos(\"%s\")\n",str));
+val=atoi_64(str);
+initial_window_xpos=(val<=0)?-1:val;
+return(0);
+}
+
 static int f_initial_window_y(char *str)
 {
 int val;
@@ -218,6 +236,15 @@ initial_window_y=(val<=0)?-1:val;
 return(0);
 }
 
+static int f_initial_window_ypos(char *str)
+{
+int val;
+DEBUG(printf("f_initial_window_ypos(\"%s\")\n",str));
+val=atoi_64(str);
+initial_window_ypos=(val<=0)?-1:val;
+return(0);
+}
+
 static int f_left_justify_sigs(char *str)
 {
 DEBUG(printf("f_left_justify_sigs(\"%s\")\n",str));
@@ -435,6 +462,7 @@ static struct rc_entry rcitems[]=
 { "color_xfill", f_color_xfill },
 { "constant_marker_update", f_constant_marker_update },
 { "convert_to_reals", f_convert_to_reals },
+{ "cursor_snap", f_cursor_snap },
 { "disable_tooltips", f_disable_tooltips },
 { "do_initial_zoom_fit", f_do_initial_zoom_fit },
 { "dynamic_resizing", f_dynamic_resizing },
@@ -450,7 +478,9 @@ static struct rc_entry rcitems[]=
 { "hier_max_level", f_hier_max_level },
 { "hpane_pack", f_hpane_pack },
 { "initial_window_x", f_initial_window_x },
+{ "initial_window_xpos", f_initial_window_xpos },
 { "initial_window_y", f_initial_window_y },
+{ "initial_window_ypos", f_initial_window_ypos },
 { "left_justify_sigs", f_left_justify_sigs },
 { "lxt_clock_compress_to_z", f_lxt_clock_compress_to_z },
 { "page_divisor", f_page_divisor },
@@ -473,10 +503,15 @@ static struct rc_entry rcitems[]=
 
 
 
-void read_rc_file(void)
+void read_rc_file(char *override_rc)
 {
 FILE *handle;
 
+if((override_rc)&&((handle=fopen(override_rc,"rb"))))
+	{
+	/* good, we have a handle */
+	}
+else
 #ifndef _MSC_VER
 if(!(handle=fopen(rcname,"rb")))
 	{
