@@ -11,6 +11,7 @@
 #include "analyzer.h"
 #include "aet.h"
 #include "vcd.h"
+#include "lx2.h"
 #include "debug.h"
 
 static GtkWidget *window1;
@@ -21,6 +22,11 @@ SearchProgressData *pdata;
 
 static int is_active=0;
 
+static char is_insert_running = 0;
+static char is_replace_running = 0;
+static char is_append_running = 0;
+static char is_searching_running = 0;
+
 
 int searchbox_is_active(void)
 {
@@ -46,10 +52,10 @@ static void enter_callback_e(GtkWidget *widget, GtkWidget *nothing)
 
 static void destroy_callback_e(GtkWidget *widget, GtkWidget *nothing)
 {
-  DEBUG(printf("Entry Cancel\n"));
-  entrybox_text_local=NULL;
-  gtk_grab_remove(window1);
-  gtk_widget_destroy(window1);
+DEBUG(printf("Entry Cancel\n"));
+entrybox_text_local=NULL;
+gtk_grab_remove(window1);
+gtk_widget_destroy(window1);
 }
 
 static void entrybox_local(char *title, int width, char *default_text, int maxch, GtkSignalFunc func)
@@ -138,7 +144,9 @@ DEBUG(printf("picked: %s\n", regex_name[regex_which]));
 static GtkWidget *window;
 static GtkWidget *entry;
 static GtkWidget *clist;
-static char *searchbox_text="";
+static char default_null_searchbox_text = 0;
+static char *searchbox_text=&default_null_searchbox_text;
+/*static char *searchbox_text="";*/
 static char bundle_direction=0;
 static GtkSignalFunc cleanup;
 static int num_rows=0;
@@ -203,7 +211,6 @@ Traces tcache;
 struct symchain *symc, *symc_current;
 int i;
 
-static char is_insert_running = 0;
 gfloat interval;
 
 if(is_insert_running) return;
@@ -219,6 +226,48 @@ traces.first=traces.last=NULL;
 GTK_ADJUSTMENT(pdata->adj)->upper = (gfloat)((num_rows>1)?num_rows-1:1);
 interval = (gfloat)(num_rows/100.0);
 
+/* LX2 */
+if(is_lx2)
+	{
+	int pre_import=0;
+	
+	for(i=0;i<num_rows;i++)
+		{
+		struct symbol *s, *t;
+		s=(struct symbol *)gtk_clist_get_row_data(GTK_CLIST(clist), i);
+		if(s->selected)
+			{
+			if((!s->vec_root)||(!autocoalesce))
+				{
+	                        if(s->n->mvlfac)
+	                                {
+	                                lx2_set_fac_process_mask(s->n);
+	                                pre_import++;
+	                                }
+				}
+				else
+				{
+				t=s->vec_root;
+				while(t)
+					{
+                                        if(t->n->mvlfac)
+                                                {
+                                                lx2_set_fac_process_mask(t->n);
+                                                pre_import++;
+                                                }
+					t=t->vec_chain;
+					}
+				}
+			}
+		}
+
+        if(pre_import)
+                {
+                lx2_import_masked();
+                }
+	}
+/* LX2 */
+
 for(i=0;i<num_rows;i++)
 	{
 	int len;
@@ -299,7 +348,6 @@ int i;
 Trptr tfirst, tlast;
 struct symchain *symc, *symc_current;
 
-static char is_replace_running = 0;
 gfloat interval;
 
 if(is_replace_running) return;
@@ -317,6 +365,48 @@ GTK_ADJUSTMENT(pdata->adj)->upper = (gfloat)((num_rows>1)?num_rows-1:1);
 interval = (gfloat)(num_rows/100.0);
 pdata->oldvalue = -1.0;
 
+/* LX2 */
+if(is_lx2)
+	{
+	int pre_import=0;
+	
+	for(i=0;i<num_rows;i++)
+		{
+		struct symbol *s, *t;
+		s=(struct symbol *)gtk_clist_get_row_data(GTK_CLIST(clist), i);
+		if(s->selected)
+			{
+			if((!s->vec_root)||(!autocoalesce))
+				{
+	                        if(s->n->mvlfac)
+	                                {
+	                                lx2_set_fac_process_mask(s->n);
+	                                pre_import++;
+	                                }
+				}
+				else
+				{
+				t=s->vec_root;
+				while(t)
+					{
+                                        if(t->n->mvlfac)
+                                                {
+                                                lx2_set_fac_process_mask(t->n);
+                                                pre_import++;
+                                                }
+					t=t->vec_chain;
+					}
+				}
+			}
+		}
+
+        if(pre_import)
+                {
+                lx2_import_masked();
+                }
+	}
+/* LX2 */
+
 for(i=0;i<num_rows;i++)
 	{
 	int len;
@@ -405,7 +495,6 @@ static void ok_callback(GtkWidget *widget, GtkWidget *nothing)
 int i;
 struct symchain *symc, *symc_current;
 
-static char is_append_running = 0;
 gfloat interval;
 
 if(is_append_running) return;
@@ -418,6 +507,48 @@ GTK_ADJUSTMENT(pdata->adj)->upper = (gfloat)((num_rows>1)?num_rows-1:1);
 interval = (gfloat)(num_rows/100.0);
 pdata->oldvalue = -1.0;
 
+/* LX2 */
+if(is_lx2)
+	{
+	int pre_import=0;
+	
+	for(i=0;i<num_rows;i++)
+		{
+		struct symbol *s, *t;
+		s=(struct symbol *)gtk_clist_get_row_data(GTK_CLIST(clist), i);
+		if(s->selected)
+			{
+			if((!s->vec_root)||(!autocoalesce))
+				{
+	                        if(s->n->mvlfac)
+	                                {
+	                                lx2_set_fac_process_mask(s->n);
+	                                pre_import++;
+	                                }
+				}
+				else
+				{
+				t=s->vec_root;
+				while(t)
+					{
+                                        if(t->n->mvlfac)
+                                                {
+                                                lx2_set_fac_process_mask(t->n);
+                                                pre_import++;
+                                                }
+					t=t->vec_chain;
+					}
+				}
+			}
+		}
+
+        if(pre_import)
+                {
+                lx2_import_masked();
+                }
+	}
+/* LX2 */
+
 for(i=0;i<num_rows;i++)
 	{
 	int len;
@@ -511,7 +642,6 @@ int i, j, k, row;
 int pixlen, maxpixlen;
 char *s, *tmp2;
 
-static char is_searching_running = 0;
 gfloat interval;
 
 if(is_searching_running) return;
@@ -522,7 +652,8 @@ pixlen=0; maxpixlen=0;
 
 entry_text = gtk_entry_get_text(GTK_ENTRY(entry));
 DEBUG(printf("Entry contents: %s\n", entry_text));
-if(!(len=strlen(entry_text))) searchbox_text=NULL;
+/* if(!(len=strlen(entry_text))) searchbox_text=NULL; */
+if(!(len=strlen(entry_text))) searchbox_text=&default_null_searchbox_text;
       else strcpy((searchbox_text=(char *)malloc_2(len+1)),entry_text);
 
 num_rows=0;
@@ -611,8 +742,11 @@ if(num_rows>=WAVE_MAX_CLIST_LENGTH)
 
 static void destroy_callback(GtkWidget *widget, GtkWidget *nothing)
 {
-  is_active=0;
-  gtk_widget_destroy(window);
+if((!is_insert_running)&&(!is_replace_running)&&(!is_append_running)&&(!is_searching_running))
+	{
+  	is_active=0;
+  	gtk_widget_destroy(window);
+	}
 }
 
 
