@@ -7,11 +7,12 @@
  * of the License, or (at your option) any later version.
  */
 
+#include "gtk12compat.h"
 #include <gtk/gtk.h>
 #include "debug.h"
 
 static GtkWidget *window;
-static GtkSignalFunc cleanup;
+static void (*cleanup)(GtkWidget *, gpointer);
 
 static void ok_callback(GtkWidget *widget, GtkWidget *nothing)
 {
@@ -36,7 +37,7 @@ void simplereqbox(char *title, int width, char *default_text,
     GtkWidget *button1, *button2;
     GtkWidget *label, *separator;
 
-    cleanup=func;
+    cleanup=WAVE_GTK_SFUNCAST(func);
 
     /* create a new modal window */
     window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
