@@ -9,6 +9,8 @@
 
 #include <gtk/gtk.h>
 #include "aet.h"
+#include "lxt2_read.h"
+#include "lx2.h"
    
 /* Add some text to our text widget - this is a callback that is invoked
 when our window is realized. We could also force our window to be
@@ -37,6 +39,11 @@ if(is_lxt)
 	{
 	status_text("LXT loaded successfully.\n");
 	}
+else
+if(is_lx2)
+	{
+	status_text("LXT2 loaded successfully.\n");
+	}
 
 sprintf(buf,"[%d] facilities found.\n",numfacs);
 status_text(buf);
@@ -77,7 +84,7 @@ gtk_widget_set_usize(GTK_WIDGET(text), 100, 50);
 gtk_widget_show (text);
 
 /* And a VScrollbar in the upper right */
-vscrollbar = gtk_vscrollbar_new (GTK_TEXT (text)->vadj);
+vscrollbar = gtk_vscrollbar_new ((GTK_TEXT (text))->vadj);
 gtk_table_attach (GTK_TABLE (table), vscrollbar, 15, 16, 0, 1,
 			GTK_FILL, GTK_FILL | GTK_SHRINK | GTK_EXPAND, 0, 0);
 gtk_widget_show (vscrollbar);
