@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 1999.
+ * Copyright (c) Tony Bybell 1999-2003.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,12 @@
 TimeType *timearray=NULL;
 int timearray_size=0;
 
+static GtkWidget *ptr_mark_count_label=NULL;   /* pointer to mark count label on pattern search dialog */
 
 struct strace *straces=NULL;   /* when active, disables certain ops! */
+struct strace *shadow_straces=NULL;
+
+struct strace_defer_free *strace_defer_free_head=NULL;
 
 static GtkWidget *window;
 static GtkSignalFunc cleanup;
@@ -21,14 +25,15 @@ static GtkSignalFunc cleanup;
 static char *logical[]=
 	{"AND", "OR", "XOR", "NAND", "NOR", "XNOR"};
 
-enum st_stype
-	{ST_DC, ST_HIGH, ST_MID, ST_X, ST_LOW, ST_STRING,
-	 ST_RISE, ST_FALL, ST_ANY};
 static char *stype[]=
 	{"Don't Care", "High", "Z (Mid)", "X", "Low", "String",
 	 "Rising Edge", "Falling Edge", "Any Edge"};
 
-static char logical_mutex[6]={0,0,0,0,0,0};
+char logical_mutex[6]={0,0,0,0,0,0};
+char shadow_logical_mutex[6]={0,0,0,0,0,0};
+char shadow_active = 0;
+char shadow_type = 0;
+char *shadow_string = NULL;
 
 
 /*
@@ -47,14 +52,6 @@ return(res);
 }
 
 
-/*
- * complain about certain ops conflict with strace...
- */
-void strace_error(void)
-{
-status_text("Can't add/delete traces when waveform display search is active!\n");
-}
-
 /*
  * free the straces...
  */
@@ -62,6 +59,7 @@ static void free_straces(void)
 {
 struct strace *s, *skill;
 int i;
+struct strace_defer_free *sd, *sd2;
 
 s=straces;
 
@@ -78,6 +76,17 @@ while(s)
 	}
 
 straces=NULL;
+
+sd = strace_defer_free_head;
+
+while(sd)
+	{
+	FreeTrace(sd->defer);
+	sd2 = sd->next;
+	free_2(sd);
+	sd = sd2;
+	}
+strace_defer_free_head = NULL;
 }
 
 
@@ -153,7 +162,13 @@ strace_search(STRACE_BACKWARD);
 static void mark_callback(GtkWidget *widget, GtkWidget *nothing)
 {
 DEBUG(printf("Marking..\n"));
+if(shadow_straces)
+	{
+	delete_strace_context();
+	}
+
 strace_maketimetrace(1);
+cache_actual_pattern_mark_traces();
 
 MaxSignalLength();
 signalarea_configure_event(signalarea, NULL);
@@ -163,6 +178,10 @@ wavearea_configure_event(wavearea, NULL);
 static void clear_callback(GtkWidget *widget, GtkWidget *nothing)
 {
 DEBUG(printf("Clearing..\n"));
+if(shadow_straces)
+	{
+	delete_strace_context();
+	}
 strace_maketimetrace(0);
 
 MaxSignalLength();
@@ -173,15 +192,31 @@ wavearea_configure_event(wavearea, NULL);
 static void destroy_callback(GtkWidget *widget, GtkWidget *nothing)
 {
   free_straces();
+  ptr_mark_count_label=NULL;
   gtk_widget_destroy(window);
 }
 
+/* update mark count label on pattern search dialog */
+
+static void update_mark_count_label()
+{
+if(ptr_mark_count_label)
+    {
+    char mark_count_buf[64];
+    if (timearray_size < 1)
+	mark_count_buf[0] = 0;
+    else
+	sprintf (mark_count_buf, "Mark Count: %d", timearray_size>=1 ? timearray_size-1 : 0);
+    gtk_label_set_text (GTK_LABEL(ptr_mark_count_label), mark_count_buf);
+    }
+}
+
 void tracesearchbox(char *title, GtkSignalFunc func)
 {
     GtkWidget *menu, *menuitem, *optionmenu;
     GSList *group; 
     GtkWidget *entry;
-    GtkWidget *vbox, *hbox, *small_hbox, *vbox_g, *label;
+    GtkWidget *vbox, *hbox, *small_hbox, *count_hbox, *vbox_g, *label;
     GtkWidget *button1, *button1a, *button1b, *button1c, *button2, *scrolled_win, *frame, *separator;
     Trptr t;
     int i;
@@ -193,12 +228,6 @@ void tracesearchbox(char *title, GtkSignalFunc func)
 	return; /* is already active */
 	}
 
-    if((searchbox_is_active())||(treebox_is_active())||(hier_searchbox_is_active()))
-	{
-	status_text("Can't activate waveform display search when signal searches are active\n");
-	return;
-	}
-
     cleanup=func;
     numtraces=0;
 
@@ -330,6 +359,18 @@ void tracesearchbox(char *title, GtkSignalFunc func)
 
     gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled_win), vbox_g);
 
+    do		/* add a horizontal box to display the mark count */
+	{
+	count_hbox = gtk_hbox_new (TRUE, 0);
+	gtk_widget_show (count_hbox);
+	gtk_box_pack_start (GTK_BOX (vbox), count_hbox, FALSE, FALSE, 0);
+
+	ptr_mark_count_label = gtk_label_new ("");
+	gtk_widget_show (ptr_mark_count_label);
+	gtk_box_pack_start (GTK_BOX (count_hbox), ptr_mark_count_label, TRUE, FALSE, 0);
+	update_mark_count_label ();
+	} while (0);
+
     hbox = gtk_hbox_new (FALSE, 1);
     gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
     gtk_widget_show (hbox);
@@ -489,7 +530,7 @@ while(s)
 		TimeType  tt;
 
 		h=bsearch_node(t->n.nd, basetime);
-		while(h->time==h->next->time) h=h->next;
+		while(h->next && h->time==h->next->time) h=h->next;
 		if((whichpass)||(tims.marker>=0)) h=h->next;
 		if(!h) return;
 		s->his.h=h;
@@ -503,7 +544,7 @@ while(s)
 		TimeType  tt;
 
 		v=bsearch_vector(t->n.vec, basetime);
-		while(v->time==v->next->time) v=v->next;
+		while(v->next && v->time==v->next->time) v=v->next;
 		if((whichpass)||(tims.marker>=0)) v=v->next;
 		if(!v) return;
 		s->his.v=v;
@@ -529,7 +570,7 @@ while(s)
 		if(adjust(s->his.h->time,shift_timebase)!=maxbase) 
 			{
 			s->his.h=bsearch_node(t->n.nd, maxbase);
-			while(s->his.h->time==s->his.h->next->time) s->his.h=s->his.h->next;
+			while(s->his.h->next && s->his.h->time==s->his.h->next->time) s->his.h=s->his.h->next;
 			}
 		if(t->flags&TR_INVERT)  
                 	{
@@ -607,7 +648,7 @@ while(s)
 			if(adjust(s->his.v->time,shift_timebase)!=maxbase) 
 				{
 				s->his.v=bsearch_vector(t->n.vec, maxbase);
-				while(s->his.v->time==s->his.v->next->time) s->his.v=s->his.v->next;
+				while(s->his.v->next && s->his.v->time==s->his.v->next->time) s->his.v=s->his.v->next;
 				}
 			chval=convert_ascii(t,s->his.v);
 			}
@@ -616,7 +657,7 @@ while(s)
 			if(adjust(s->his.h->time,shift_timebase)!=maxbase) 
 				{
 				s->his.h=bsearch_node(t->n.nd, maxbase);
-				while(s->his.h->time==s->his.h->next->time) s->his.h=s->his.h->next;
+				while(s->his.h->next && s->his.h->time==s->his.h->next->time) s->his.h=s->his.h->next;
 				}
 			if(s->his.h->flags&HIST_REAL)
 				{
@@ -882,7 +923,7 @@ while(s)
 		if(adjust(s->his.h->time,shift_timebase)!=maxbase) 
 			{
 			s->his.h=bsearch_node(t->n.nd, maxbase);
-			while(s->his.h->time==s->his.h->next->time) s->his.h=s->his.h->next;
+			while(s->his.h->next && s->his.h->time==s->his.h->next->time) s->his.h=s->his.h->next;
 			}
 		if(t->flags&TR_INVERT)  
                 	{
@@ -960,7 +1001,7 @@ while(s)
 			if(adjust(s->his.v->time,shift_timebase)!=maxbase) 
 				{
 				s->his.v=bsearch_vector(t->n.vec, maxbase);
-				while(s->his.v->time==s->his.v->next->time) s->his.v=s->his.v->next;
+				while(s->his.v->next && s->his.v->time==s->his.v->next->time) s->his.v=s->his.v->next;
 				}
 			chval=convert_ascii(t,s->his.v);
 			}
@@ -969,7 +1010,7 @@ while(s)
 			if(adjust(s->his.h->time,shift_timebase)!=maxbase) 
 				{
 				s->his.h=bsearch_node(t->n.nd, maxbase);
-				while(s->his.h->time==s->his.h->next->time) s->his.h=s->his.h->next;
+				while(s->his.h->next && s->his.h->time==s->his.h->next->time) s->his.h=s->his.h->next;
 				}
 			if(s->his.h->flags&HIST_REAL)
 				{
@@ -1159,7 +1200,12 @@ if(timearray)
 
 timearray_size=0;
 
-if(!mode) return;	/* merely free stuff up */
+if((!mode)&&(!shadow_active))
+	{
+	update_mark_count_label();
+	delete_mprintf();
+	return;	/* merely free stuff up */
+	}
 
 do
 	{
@@ -1187,4 +1233,218 @@ for(i=0;i<timearray_size;i++)
 	*(t++)=tchead->t;
 	tchead=tchead->next;
 	}
+
+if(!shadow_active) update_mark_count_label();
+}
+
+
+/*
+ * swap context for mark during trace load...
+ */
+void swap_strace_contexts(void)
+{
+struct strace *stemp;
+char logical_mutex_temp[6];
+
+stemp = straces;
+straces = shadow_straces;
+shadow_straces = stemp;
+
+memcpy(logical_mutex_temp, logical_mutex, 6);
+memcpy(logical_mutex, shadow_logical_mutex, 6);
+memcpy(shadow_logical_mutex, logical_mutex_temp, 6);
+}
+
+
+/*
+ * delete context
+ */
+void delete_strace_context(void)
+{
+int i;
+struct strace *stemp;
+struct strace *strace_cache;
+
+for(i=0;i<6;i++)
+	{
+	shadow_logical_mutex[i] = 0;
+	}
+
+strace_cache = straces;	/* so the trace actually deletes */
+straces=NULL;
+
+stemp = shadow_straces;
+while(stemp)
+	{
+	shadow_straces = stemp->next;
+	if(stemp->string) free_2(stemp->string);
+
+	FreeTrace(stemp->trace);
+	free_2(stemp);
+	stemp = shadow_straces;
+	}
+
+if(shadow_string)
+	{
+	free_2(shadow_string);
+	shadow_string = NULL;
+	}
+
+straces = strace_cache;
+}
+
+/*************************************************************************/
+
+/*
+ * printf to memory..
+ */
+struct mprintf_buff_t *mprintf_buff_head=NULL, *mprintf_buff_current=NULL;
+  
+int mprintf(const char *fmt, ... )
+{
+int len;
+int rc;
+va_list args;
+struct mprintf_buff_t *bt = (struct mprintf_buff_t *)calloc(1, sizeof(struct mprintf_buff_t));
+char buff[65537];
+                                
+va_start(args, fmt);
+rc=vsprintf(buff, fmt, args);
+len = strlen(buff);
+bt->str = malloc_2(len+1);
+strcpy(bt->str, buff);
+                 
+if(!mprintf_buff_current)
+        {
+        mprintf_buff_head = mprintf_buff_current = bt;
+        }
+        else
+        {
+        mprintf_buff_current->next = bt;
+        mprintf_buff_current = bt;
+        }
+                        
+return(rc);
+}
+
+/*
+ * kill mprint buffer
+ */
+void delete_mprintf(void)
+{
+if(mprintf_buff_head)
+	{
+	struct mprintf_buff_t *mb = mprintf_buff_head;
+	struct mprintf_buff_t *mbt;		
+		
+	while(mb)
+		{
+		free_2(mb->str);
+		mbt = mb->next;
+		free(mb);
+		mb = mbt;
+		}
+
+	mprintf_buff_head = mprintf_buff_current = NULL;
+	}
+}
+
+
+/*
+ * so we can (later) write out the traces which are actually marked...
+ */
+void cache_actual_pattern_mark_traces(void)
+{
+Trptr t;
+int i;
+unsigned int def=0;
+TimeType prevshift=LLDescriptor(0);
+struct strace *st;
+
+delete_mprintf();
+
+if(timearray)
+	{
+	mprintf("!%d%d%d%d%d%d\n", logical_mutex[0], logical_mutex[1], logical_mutex[2], logical_mutex[3], logical_mutex[4], logical_mutex[5]);
+	st=straces;
+
+	while(st)
+		{
+		if(st->value==ST_STRING)
+			{
+			mprintf("?\"%s\n", st->string ? st->string : ""); /* search type for this trace is string.. */
+			}
+			else
+			{
+			mprintf("?%02x\n", (unsigned char)st->value);	/* else search type for this trace.. */
+			}
+
+		t=st->trace;
+
+		if((t->flags!=def)||(st==straces))
+			{
+			mprintf("@%x\n",def=t->flags);
+			}
+
+		if((t->shift)||((prevshift)&&(!t->shift)))
+			{
+			mprintf(">"TTFormat"\n", t->shift);
+			}
+		prevshift=t->shift;
+
+		if(!(t->flags&TR_BLANK))	
+			{
+			if(t->vector)
+				{
+				int i;
+				nptr *nodes;
+
+				mprintf("#%s",t->name);
+
+				nodes=t->n.vec->bits->nodes;
+				for(i=0;i<t->n.vec->nbits;i++)
+					{
+					if(nodes[i]->expansion)
+						{
+						mprintf(" (%d)%s",nodes[i]->expansion->parentbit, nodes[i]->expansion->parent->nname);
+						}
+						else
+						{
+						mprintf(" %s",nodes[i]->nname);
+						}
+					}
+				mprintf("\n");
+				}
+				else
+				{
+				if(t->is_alias)
+					{
+					if(t->n.nd->expansion)
+						{
+						mprintf("+%s (%d)%s\n",t->name+2,t->n.nd->expansion->parentbit, t->n.nd->expansion->parent->nname);
+						}
+						else
+						{
+						mprintf("+%s %s\n",t->name+2,t->n.nd->nname);
+						}
+					}
+					else
+					{
+					if(t->n.nd->expansion)
+						{
+						mprintf("(%d)%s\n",t->n.nd->expansion->parentbit, t->n.nd->expansion->parent->nname);
+						}
+						else
+						{
+						mprintf("%s\n",t->n.nd->nname);
+						}
+					}
+				}
+			}
+
+		st=st->next;
+		} /* while(st)... */
+
+	mprintf("!!\n");	/* mark end of strace region */
+	}
 }
