@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) Tony Bybell 1999.
+ * Copyright (c) Tony Bybell 1999-2003.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,6 +12,7 @@
 
 #include <gtk/gtk.h>
 #include <string.h>
+#include <stdarg.h>
 #include "debug.h"
 #include "analyzer.h"
 #include "currenttime.h"
@@ -19,6 +20,16 @@
 
 enum strace_directions
 { STRACE_BACKWARD, STRACE_FORWARD };
+
+enum st_stype
+        {ST_DC, ST_HIGH, ST_MID, ST_X, ST_LOW, ST_STRING,
+         ST_RISE, ST_FALL, ST_ANY};
+
+struct strace_defer_free
+{
+struct strace_defer_free *next;
+Trptr defer;
+};
          
 struct strace_back
 {
@@ -51,13 +62,31 @@ TimeType t;
 };
 
 
-extern struct strace *straces;
+struct mprintf_buff_t
+{
+struct mprintf_buff_t *next;
+char *str;
+};
+
+
+extern struct strace *straces, *shadow_straces;
+extern struct strace_defer_free *strace_defer_free_head;
 extern TimeType *timearray;
 extern int timearray_size;
+extern char logical_mutex[6], shadow_logical_mutex[6];
+extern char shadow_active, shadow_type;
+extern char *shadow_string;
+extern struct mprintf_buff_t *mprintf_buff_head, *mprintf_buff_current;
 
-void strace_error(void);
 void strace_search(int direction);
 void strace_maketimetrace(int mode); /* 1=create, zero=delete */
 
+void swap_strace_contexts(void);
+void delete_strace_context(void);
+void cache_actual_pattern_mark_traces(void);
+
+int mprintf(const char *fmt, ... );
+void delete_mprintf(void);
+
 #endif
 
