@@ -12,6 +12,7 @@
 #include "tree.h"
 #include "aet.h"
 #include "vcd.h"
+#include "lx2.h"
 #include "debug.h"
 
 char autoname_bundles=0;
@@ -299,6 +300,48 @@ for(i=fetchlow(selectedtree)->which;i<=fetchhigh(selectedtree)->which;i++)
 		}
         }
 
+/* LX2 */
+if(is_lx2)
+        {
+        int pre_import = 0;
+
+        for(i=fetchlow(selectedtree)->which;i<=fetchhigh(selectedtree)->which;i++)
+                {
+                struct symbol *s, *t;
+                s=facs[i];
+                t=s->vec_root;
+                if((t)&&(autocoalesce))
+                        {
+                        if(t->selected)
+                                {
+                                while(t)
+                                        {
+                                        if(t->n->mvlfac)
+                                                {
+                                                lx2_set_fac_process_mask(t->n);
+                                                pre_import++;
+                                                }
+                                        t=t->vec_chain;
+                                        }
+                                }
+                        }
+                        else
+                        {
+                        if(s->n->mvlfac)
+                                {
+                                lx2_set_fac_process_mask(s->n);
+                                pre_import++;
+                                }
+                        }
+                }
+
+        if(pre_import)
+                {
+                lx2_import_masked();
+                }
+        }
+/* LX2 */
+
 for(i=fetchlow(selectedtree)->which;i<=fetchhigh(selectedtree)->which;i++)
         {
 	int len;
@@ -365,6 +408,48 @@ for(i=fetchlow(selectedtree)->which;i<=fetchhigh(selectedtree)->which;i++)
 		}
         }
 
+/* LX2 */
+if(is_lx2)
+        {
+        int pre_import = 0;
+
+        for(i=fetchlow(selectedtree)->which;i<=fetchhigh(selectedtree)->which;i++)
+                {
+                struct symbol *s, *t;
+                s=facs[i];
+                t=s->vec_root;
+                if((t)&&(autocoalesce))
+                        {
+                        if(t->selected)
+                                {
+                                while(t)
+                                        {
+                                        if(t->n->mvlfac)
+                                                {
+                                                lx2_set_fac_process_mask(t->n);
+                                                pre_import++;
+                                                }
+                                        t=t->vec_chain;
+                                        }
+                                }
+                        }
+                        else
+                        {
+                        if(s->n->mvlfac)
+                                {
+                                lx2_set_fac_process_mask(s->n);
+                                pre_import++;
+                                }
+                        }
+                }
+
+        if(pre_import)
+                {
+                lx2_import_masked();
+                }
+        }
+/* LX2 */
+
 for(i=fetchlow(selectedtree)->which;i<=fetchhigh(selectedtree)->which;i++)
         {
 	int len;
@@ -436,6 +521,48 @@ for(i=fetchlow(selectedtree)->which;i<=fetchhigh(selectedtree)->which;i++)
 		}
         }
 
+/* LX2 */
+if(is_lx2)
+        {
+        int pre_import = 0;
+
+        for(i=fetchlow(selectedtree)->which;i<=fetchhigh(selectedtree)->which;i++)
+                {
+                struct symbol *s, *t;
+                s=facs[i];
+                t=s->vec_root;
+                if((t)&&(autocoalesce))
+                        {
+                        if(t->selected)
+                                {
+                                while(t)
+                                        {
+                                        if(t->n->mvlfac)
+                                                {
+                                                lx2_set_fac_process_mask(t->n);
+                                                pre_import++;
+                                                }
+                                        t=t->vec_chain;
+                                        }
+                                }
+                        }
+                        else
+                        {
+                        if(s->n->mvlfac)
+                                {
+                                lx2_set_fac_process_mask(s->n);
+                                pre_import++;
+                                }
+                        }
+                }
+
+        if(pre_import)
+                {
+                lx2_import_masked();
+                }
+        }
+/* LX2 */
+
 for(i=fetchlow(selectedtree)->which;i<=fetchhigh(selectedtree)->which;i++)
         {
 	int len;
