@@ -19,6 +19,9 @@
  * fix for duplicate nets	19dec00ajb
  * support for alt hier seps	23dec00ajb
  * fix for rcs identifiers	16jan01ajb
+ * coredump fix for bad VCD	04apr02ajb
+ * min/maxid speedup            27feb03ajb
+ * bugfix on min/maxid speedup  06jul03ajb
  */
 #include "vcd.h"
 
@@ -42,6 +45,8 @@ static void vcd_cleanup(void);
 static void evcd_strcpy(char *dst, char *src);
 
 static int vcdbyteno=0;		/* really should be size_t, but this is only used for debugging mangled traces */
+static int error_count=0;	/* should always be zero */
+
 static int header_over=0;
 static int dumping_off=0;
 static TimeType start_time=-1;
@@ -55,6 +60,7 @@ static int num_glitch_regions=0;
 static char vcd_hier_delimeter[2]={0, 0};   /* fill in after rc reading code */
 
 static struct vcdsymbol *pv=NULL, *rootv=NULL;
+static char *vcdbuf=NULL, *vst=NULL, *vend=NULL;
 
 /******************************************************************/
 
@@ -106,6 +112,7 @@ static int yylen=0, yylen_cache=0;
 
 static struct vcdsymbol *vcdsymroot=NULL, *vcdsymcurr=NULL;
 static struct vcdsymbol **sorted=NULL;
+static struct vcdsymbol **indexed=NULL;
 
 enum VarTypes { V_EVENT, V_PARAMETER,
 		V_INTEGER, V_REAL, V_REG, V_SUPPLY0,
@@ -146,6 +153,28 @@ return(he_curr++);
 
 static struct queuedevent *queuedevents=NULL;
 
+/******************************************************************/
+ 
+static unsigned int vcd_minid = ~0;
+static unsigned int vcd_maxid = 0;
+
+static unsigned int vcdid_hash(char *s, int len)
+{  
+unsigned int val=0;
+int i;
+
+s+=(len-1);
+                 
+for(i=0;i<len;i++)
+        {
+        val *= 95;				/* was 94 but XL uses '!' as right hand side chars which act as leading zeros */
+        val += (((unsigned char)*s) - 32);	/* was 33 but XL ... */
+        s--;
+        }
+
+return(val);
+}
+
 /******************************************************************/
 
 /*
@@ -166,32 +195,56 @@ return(strcmp(v1, v2->id));
 /*
  * actual bsearch
  */
-static struct vcdsymbol *bsearch_vcd(char *key)
+static struct vcdsymbol *bsearch_vcd(char *key, int len)
 {
 struct vcdsymbol **v;
 struct vcdsymbol *t;
 
-v=(struct vcdsymbol **)bsearch(key, sorted, numsyms, 
-	sizeof(struct vcdsymbol *), vcdsymbsearchcompare);
+if(indexed)
+        {
+        unsigned int hsh = vcdid_hash(key, len);
+        if((hsh>=vcd_minid)&&(hsh<=vcd_maxid))
+                {
+                return(indexed[hsh-vcd_minid]);
+                }
 
-if(v)
+	return(NULL);
+        }
+
+if(sorted)
 	{
-	#ifndef VCD_BSEARCH_IS_PERFECT
-		for(;;)
-			{
-			t=*v;
-	
-			if((v==sorted)||(strcmp((*(--v))->id, key)))
+	v=(struct vcdsymbol **)bsearch(key, sorted, numsyms, 
+		sizeof(struct vcdsymbol *), vcdsymbsearchcompare);
+
+	if(v)
+		{
+		#ifndef VCD_BSEARCH_IS_PERFECT
+			for(;;)
 				{
-				return(t);
+				t=*v;
+		
+				if((v==sorted)||(strcmp((*(--v))->id, key)))
+					{
+					return(t);
+					}
 				}
-			}
-	#else
-		return(*v);
-	#endif
+		#else
+			return(*v);
+		#endif
+		}
+		else
+		{
+		return(NULL);
+		}
 	}
 	else
 	{
+	static int err = 0;
+	if(!err)
+		{
+		fprintf(stderr, "Near byte %d, VCD search table NULL..is this a VCD file?\n", vcdbyteno+(vst-vcdbuf));
+		err=1;
+		}
 	return(NULL);
 	}
 }
@@ -218,23 +271,50 @@ static void create_sorted_table(void)
 {
 struct vcdsymbol *v;
 struct vcdsymbol **pnt;
+unsigned int vcd_distance;
+int i;
 
 if(sorted) 
 	{
 	free_2(sorted);	/* this means we saw a 2nd enddefinition chunk! */
+	sorted=NULL;
+	}
+
+if(indexed)
+	{
+	free_2(indexed);
+	indexed=NULL;
 	}
 
 if(numsyms)
 	{
-	pnt=sorted=(struct vcdsymbol **)calloc_2(numsyms, sizeof(struct vcdsymbol *));
-	v=vcdsymroot;
-	while(v)
-		{
-		*(pnt++)=v;
-		v=v->next;
-		}
+        vcd_distance = vcd_maxid - vcd_minid + 1;
 
-	qsort(sorted, numsyms, sizeof(struct vcdsymbol *), vcdsymcompare);
+        if(vcd_distance <= VCD_INDEXSIZ)
+                {
+                indexed = (struct vcdsymbol **)calloc_2(vcd_distance, sizeof(struct vcdsymbol *));
+         
+		/* printf("%d symbols span ID range of %d, using indexing...\n", numsyms, vcd_distance); */
+
+                v=vcdsymroot;
+                while(v)
+                        {
+                        if(!indexed[v->nid - vcd_minid]) indexed[v->nid - vcd_minid] = v;
+                        v=v->next;
+                        }
+                }
+                else
+		{	
+		pnt=sorted=(struct vcdsymbol **)calloc_2(numsyms, sizeof(struct vcdsymbol *));
+		v=vcdsymroot;
+		while(v)
+			{
+			*(pnt++)=v;
+			v=v->next;
+			}
+	
+		qsort(sorted, numsyms, sizeof(struct vcdsymbol *), vcdsymcompare);
+		}
 	}
 }
 
@@ -243,8 +323,6 @@ if(numsyms)
 /*
  * single char get inlined/optimized
  */
-static char *vcdbuf=NULL, *vst=NULL, *vend=NULL;
-
 static void getch_alloc(void)
 {
 vend=vst=vcdbuf=(char *)calloc_2(1,VCD_BSIZ);
@@ -628,7 +706,7 @@ switch(yytext[0])
 	case 'Z':
 		if(yylen>1)
 			{
-			v=bsearch_vcd(yytext+1);	
+			v=bsearch_vcd(yytext+1, yylen-1);	
 			if(!v)
 				{
 				fprintf(stderr,"Near byte %d, Unknown VCD identifier: '%s'\n",vcdbyteno+(vst-vcdbuf),yytext+1);
@@ -671,7 +749,7 @@ switch(yytext[0])
 		vlen=yylen-1;
 
 		get_strtoken();
-		v=bsearch_vcd(yytext);	
+		v=bsearch_vcd(yytext, yylen);	
 		if(!v)
 			{
 			fprintf(stderr,"Near byte %d, Unknown identifier: '%s'\n",vcdbyteno+(vst-vcdbuf), yytext);
@@ -679,7 +757,7 @@ switch(yytext[0])
 			}
 			else
 			{
-			if((convert_to_reals)&&(v->vartype==V_REAL))
+			if ((v->vartype==V_REAL)||((convert_to_reals)&&((v->vartype==V_INTEGER)||(v->vartype==V_PARAMETER))))
 				{
 				double *d;
 				char *pnt;
@@ -765,7 +843,7 @@ switch(yytext[0])
 		get_strtoken();	/* throw away 0_strength_component */
 		get_strtoken(); /* throw away 0_strength_component */
 		get_strtoken(); /* this is the id                  */
-		v=bsearch_vcd(yytext);	
+		v=bsearch_vcd(yytext, yylen);	
 		if(!v)
 			{
 			fprintf(stderr,"Near byte %d, Unknown identifier: '%s'\n",vcdbyteno+(vst-vcdbuf), yytext);
@@ -773,7 +851,7 @@ switch(yytext[0])
 			}
 			else
 			{
-			if((convert_to_reals)&&(v->vartype==V_REAL))	/* should never happen, but just in case.. */
+			if ((v->vartype==V_REAL)||((convert_to_reals)&&((v->vartype==V_INTEGER)||(v->vartype==V_PARAMETER))))
 				{
 				double *d;
 				char *pnt;
@@ -858,7 +936,7 @@ switch(yytext[0])
 		sscanf(yytext+1,"%lg",d);
 		
 		get_strtoken();
-		v=bsearch_vcd(yytext);	
+		v=bsearch_vcd(yytext, yylen);	
 		if(!v)
 			{
 			fprintf(stderr,"Near byte %d, Unknown identifier: '%s'\n",vcdbyteno+(vst-vcdbuf), yytext);
@@ -882,7 +960,7 @@ switch(yytext[0])
 		strcpy(d, yytext+1);
 		
 		get_strtoken();
-		v=bsearch_vcd(yytext);	
+		v=bsearch_vcd(yytext, yylen);	
 		if(!v)
 			{
 			fprintf(stderr,"Near byte %d, Unknown identifier: '%s'\n",vcdbyteno+(vst-vcdbuf), yytext);
@@ -1114,6 +1192,10 @@ for(;;)
 				if(vtok==V_END) goto err;
 				v->id=(char *)malloc_2(yylen+1);
 				strcpy(v->id, yytext);
+                                v->nid=vcdid_hash(yytext,yylen);
+
+                                if(v->nid < vcd_minid) vcd_minid = v->nid;
+                                if(v->nid > vcd_maxid) vcd_maxid = v->nid;
 
 				vtok=get_vartoken();
 				if(vtok!=V_STRING) goto err;
@@ -1173,6 +1255,10 @@ for(;;)
 				if(vtok==V_END) goto err;
 				v->id=(char *)malloc_2(yylen+1);
 				strcpy(v->id, yytext);
+                                v->nid=vcdid_hash(yytext,yylen);
+                                
+                                if(v->nid < vcd_minid) vcd_minid = v->nid;
+                                if(v->nid > vcd_maxid) vcd_maxid = v->nid;
 
 				vtok=get_vartoken();
 				if(vtok!=V_STRING) goto err;
@@ -1392,7 +1478,16 @@ for(;;)
 			err:
 			if(v)
 				{
-				if(v->name) free_2(v->name);
+				error_count++;
+				if(v->name) 
+					{
+					fprintf(stderr, "Near byte %d, $VAR parse error encountered with '%s'\n", vcdbyteno+(vst-vcdbuf), v->name);
+					free_2(v->name);
+					}
+					else
+					{
+					fprintf(stderr, "Near byte %d, $VAR parse error encountered\n", vcdbyteno+(vst-vcdbuf));
+					}
 				if(v->id) free_2(v->id);
 				if(v->value) free_2(v->value);
 				free_2(v);
@@ -1405,18 +1500,23 @@ for(;;)
 		case T_ENDDEFINITIONS:
 			header_over=1;	/* do symbol table management here */
 			create_sorted_table();
-			if(!sorted)
+			if((!sorted)&&(!indexed))
 				{
 				fprintf(stderr, "No symbols in VCD file..nothing to do!\n");
 				exit(1);
 				}
+			if(error_count)
+				{
+				fprintf(stderr, "\n%d VCD parse errors encountered, exiting.\n", error_count);
+				exit(1);
+				}
 			break;
 		case T_STRING:
 			if(!header_over)
 				{
 				header_over=1;	/* do symbol table management here */
 				create_sorted_table();
-				if(!sorted) break;
+				if((!sorted)&&(!indexed)) break;
 				}
 				{
 				/* catchall for events when header over */
@@ -1446,10 +1546,22 @@ for(;;)
 		case T_DUMPOFF:
 		case T_DUMPPORTSOFF:
 			dumping_off=1;
+			if((!blackout_regions)||((blackout_regions)&&(blackout_regions->bstart<=blackout_regions->bend)))
+				{
+				struct blackout_region_t *bt = calloc(1, sizeof(struct blackout_region_t));
+
+				bt->bstart = current_time;
+				bt->next = blackout_regions;
+				blackout_regions = bt;
+				}
 			break;
 		case T_DUMPON:
 		case T_DUMPPORTSON:
 			dumping_off=0;
+			if((blackout_regions)&&(blackout_regions->bstart>blackout_regions->bend))
+				{
+				blackout_regions->bend = current_time;
+				}
 			break;
 		case T_DUMPVARS:
 		case T_DUMPPORTS:
@@ -1464,6 +1576,10 @@ for(;;)
 			sync_end(NULL);	/* skip over unknown keywords */
 			break;
 		case T_EOF:
+			if((blackout_regions)&&(blackout_regions->bstart>blackout_regions->bend))
+				{
+				blackout_regions->bend = current_time;
+				}
 			return;
 		default:
 			DEBUG(fprintf(stderr,"UNKNOWN TOKEN\n"));
@@ -1700,14 +1816,13 @@ switch(ch)
 static void add_tail_histents(void)
 {
 int i,j;
+struct vcdsymbol *v;
 
 /* dump out any pending events 1st */
 struct queuedevent *q;
 q=queuedevents;
 while(q)
 	{	
-	struct vcdsymbol *v;
-
 	v=q->sym;
 	if(current_time!=(v->ev->last_event_time+1))
 		{
@@ -1720,10 +1835,9 @@ while(q)
 
 /* then do 'x' trailers */
 
-for(i=0;i<numsyms;i++)
+v=vcdsymroot;
+while(v)
 	{
-	struct vcdsymbol *v;
-	v=sorted[i];
 	if(v->vartype==V_REAL)
 		{
 		double *d;
@@ -1742,13 +1856,13 @@ for(i=0;i<numsyms;i++)
 		{
 		add_histent(MAX_HISTENT_TIME-1, v->narray[0], 'x', 0, (char *)calloc_2(1,sizeof(char)));
 		}
+
+	v=v->next;
 	}
 
-
-for(i=0;i<numsyms;i++)
+v=vcdsymroot;
+while(v)
 	{
-	struct vcdsymbol *v;
-	v=sorted[i];
 	if(v->vartype==V_REAL)
 		{
 		double *d;
@@ -1767,6 +1881,8 @@ for(i=0;i<numsyms;i++)
 		{
 		add_histent(MAX_HISTENT_TIME, v->narray[0], 'z', 0, (char *)calloc_2(1,sizeof(char)));
 		}
+
+	v=v->next;
 	}
 }
 
@@ -1779,10 +1895,11 @@ int max_slen=-1;
 struct sym_chain *sym_chain=NULL, *sym_curr=NULL;
 int duphier=0;
 char hashdirty;
+struct vcdsymbol *v, *vprime;
 
-for(i=0;i<numsyms;i++)
+v=vcdsymroot;
+while(v)
 	{
-	struct vcdsymbol *v, *vprime;
 	int msi;
 	int delta;
 
@@ -1791,7 +1908,6 @@ for(i=0;i<numsyms;i++)
 		int slen;
 		int substnode;
 
-		v=sorted[i];
 		msi=v->msi;
 		delta=((v->lsi-v->msi)<0)?-1:1;
 		substnode=0;
@@ -1806,7 +1922,7 @@ for(i=0;i<numsyms;i++)
 			slen++;
 			}
 
-		if((vprime=bsearch_vcd(v->id))!=v) /* hash mish means dup net */
+		if((vprime=bsearch_vcd(v->id, strlen(v->id)))!=v) /* hash mish means dup net */
 			{
 			if(v->size!=vprime->size)
 				{
@@ -1974,6 +2090,8 @@ for(i=0;i<numsyms;i++)
 				}
 			}
 		}
+
+	v=v->next;
 	}
 
 if(sym_chain)
@@ -2019,7 +2137,7 @@ for(i=0;i<numfacs;i++)
         }
 
 /* quicksort(facs,0,numfacs-1); */	/* quicksort deprecated because it degenerates on sorted traces..badly.  very badly. */
-heapsort(facs,numfacs);
+wave_heapsort(facs,numfacs);
 
 for(i=0;i<numfacs;i++)
         {
@@ -2053,26 +2171,32 @@ static void vcd_cleanup(void)
 {
 int i;
 struct slist *s, *s2;
+struct vcdsymbol *v, *vt;
+
+if(indexed)
+	{
+	free_2(indexed); indexed=NULL; 
+	}
 
 if(sorted)
 	{
-	for(i=0;i<numsyms;i++)
-		{
-		struct vcdsymbol *v;
-		v=sorted[i];
-		if(v)
-			{
-			if(v->name) free_2(v->name);
-			if(v->id) free_2(v->id);
-			if(v->value) free_2(v->value);
-			if(v->ev) free_2(v->ev);
-			if(v->narray) free_2(v->narray);
-			free_2(v);
-			}
-		}
 	free_2(sorted); sorted=NULL; 
 	}
 
+v=vcdsymroot;
+while(v)
+	{
+	if(v->name) free_2(v->name);
+	if(v->id) free_2(v->id);
+	if(v->value) free_2(v->value);
+	if(v->ev) free_2(v->ev);
+	if(v->narray) free_2(v->narray);
+	vt=v;
+	v=v->next;
+	free_2(vt);
+	}
+vcdsymroot=vcdsymcurr=NULL;
+
 if(slisthier) { free_2(slisthier); slisthier=NULL; }
 s=slistroot;
 while(s)
@@ -2157,7 +2281,7 @@ getch_alloc();		/* alloc membuff for vcd getch buffer */
 
 build_slisthier();
 vcd_parse();
-if(!sorted)
+if((!sorted)&&(!indexed))
 	{
 	fprintf(stderr, "No symbols in VCD file..is it malformed?  Exiting!\n");
 	exit(1);
@@ -2166,7 +2290,7 @@ add_tail_histents();
 
 if(vcd_save_handle) fclose(vcd_save_handle);
 
-printf("["TTFormat"] start time.\n["TTFormat"] end time.\n", start_time, end_time);
+printf("["TTFormat"] start time.\n["TTFormat"] end time.\n", start_time*time_scale, end_time*time_scale);
 if(num_glitches) printf("Warning: encountered %d glitch%s across %d glitch region%s.\n", 
 		num_glitches, (num_glitches!=1)?"es":"",
 		num_glitch_regions, (num_glitch_regions!=1)?"s":"");
