@@ -7,6 +7,7 @@
  * of the License, or (at your option) any later version.
  */
 
+#include "gtk12compat.h"
 #include "currenttime.h"
 #include "pixmaps.h"
 #include "aet.h"
@@ -22,7 +23,6 @@ static void draw_vptr_trace(Trptr t, vptr v, int which);
 static void rendertraces(void);
 static void rendertimes(void);
 
-
 static gint m1x, m2x;
 
 char signalwindow_width_dirty=1;	/* prime it up for the 1st one at least.. */
@@ -39,9 +39,11 @@ int vector_padding=4;
 static int in_button_press=0;
 char left_justify_sigs=0;
 char zoom_pow10_snap=0;
+int cursor_snap=0;
 
 gfloat old_wvalue=-1.0;
 
+struct blackout_region_t *blackout_regions = NULL;
 TimeType zoom=0, scale=1, nsperframe=1, pixelsperframe=1, hashstep=1;
 static TimeType prevtim=-1;
 
@@ -84,6 +86,83 @@ static const GdkModifierType   bmask[4]= {0, GDK_BUTTON1_MASK, 0, GDK_BUTTON3_MA
 static const GdkModifierType m_bmask[4]= {0, GDK_BUTTON1_MOTION_MASK, 0, GDK_BUTTON3_MOTION_MASK };	/* button 1, 3 motion encodings */
 
 
+/*
+ * aldec-like "snap" feature...
+ */
+TimeType cook_markertime(TimeType marker, gint x, gint y)
+{
+int i, num_traces_displayable;                
+Trptr t;
+TimeType lft, rgh;
+char lftinv, rghinv;
+gdouble xlft, xrgh;
+gdouble xlftd, xrghd;
+
+if(!cursor_snap) return(marker);
+
+num_traces_displayable=wavearea->allocation.height/(fontheight);
+num_traces_displayable--;   /* for the time trace that is always there */
+
+y-=fontheight;
+if(y<0) y=0;
+y/=fontheight;		    /* y now indicates the trace in question */
+if(y>num_traces_displayable) y=num_traces_displayable;
+
+t=topmost_trace;
+for(i=0;i<y;i++)
+	{
+	if(!t) return(marker);
+	t=t->next;
+	}
+
+if((!t)||(t->flags&(TR_BLANK|TR_EXCLUDE))) return(marker);
+
+if(t->vector)
+	{
+	vptr v = bsearch_vector(t->n.vec, marker - t->shift);
+	vptr v2 = v ? v->next : NULL;
+
+	if((!v)||(!v2)) return(marker);	/* should never happen */
+
+	lft = v->time;
+	rgh = v2->time;
+	}
+	else
+	{
+	hptr h = bsearch_node(t->n.nd, marker - t->shift);
+	hptr h2 = h ? h->next : NULL;
+
+	if((!h)||(!h2)) return(marker);	/* should never happen */
+
+	lft = h->time;
+	rgh = h2->time;
+	}
+
+lftinv = (lft < (tims.start - t->shift))||(lft >= (tims.end - t->shift))||(lft >= (tims.last - t->shift));
+rghinv = (rgh < (tims.start - t->shift))||(rgh >= (tims.end - t->shift))||(rgh >= (tims.last - t->shift));
+
+xlft = (lft + t->shift - tims.start) * pxns;
+xrgh = (rgh + t->shift - tims.start) * pxns;
+
+xlftd = xlft - x;
+if(xlftd<(gdouble)0.0) xlftd = ((gdouble)0.0) - xlftd;
+
+xrghd = xrgh - x;
+if(xrghd<(gdouble)0.0) xrghd = ((gdouble)0.0) - xrghd;
+
+if(xlftd<=xrghd)
+	{
+	if((!lftinv)&&(xlftd<=cursor_snap)) marker = lft + t->shift;
+	}
+	else
+	{
+	if((!rghinv)&&(xrghd<=cursor_snap)) marker = rgh + t->shift;
+	}
+
+return(marker);
+}
+
+
 static void draw_named_markers(void)
 {
 gdouble pixstep;
@@ -257,7 +336,42 @@ if(m1x==m2x)
 	}
 }
 
+static void renderblackout(void)
+{
+gfloat pageinc;
+TimeType lhs, rhs, lclip, rclip;
+struct blackout_region_t *bt = blackout_regions;
 
+if(bt)
+	{
+	pageinc=(gfloat)(((gdouble)wavewidth)*nspx);
+	lhs = tims.start;
+	rhs = pageinc + lhs;
+
+	while(bt)
+		{
+		if( ((bt->bstart <= lhs) && (bt->bend >= lhs)) || ((bt->bstart >= lhs) && (bt->bstart <= rhs)) )
+			{
+			lclip = bt->bstart; rclip = bt->bend;
+
+			if(lclip < lhs) lclip = lhs;
+				else if (lclip > rhs) lclip = rhs;
+
+			if(rclip < lhs) rclip = lhs;
+
+			lclip -= lhs;
+			rclip -= lhs;
+			if(rclip>(wavewidth+1)) rclip = wavewidth+1;
+			
+			gdk_draw_rectangle(wavepixmap, gc_xfill, TRUE, 
+				(((gdouble)lclip)*pxns), fontheight,
+				(((gdouble)(rclip-lclip))*pxns), waveheight-fontheight);
+			}
+
+		bt=bt->next;
+		}
+	}
+}
 
 static void     
 service_hslider(GtkWidget *text, gpointer data)
@@ -341,7 +455,7 @@ gdk_draw_pixmap(signalarea->window, signalarea->style->fg_gc[GTK_WIDGET_STATE(si
         signalarea->allocation.width, signalarea->allocation.height);
 }
 
-static void button_motion_common(gint xin, int pressrel) 
+static void button_motion_common(gint xin, gint yin, int pressrel) 
 {
 gdouble x,offset,pixstep;
 TimeType newcurr;
@@ -371,7 +485,7 @@ if(newcurr<tims.start)
 	newcurr=tims.start;
 	}
 
-update_markertime(tims.marker=time_trunc(newcurr));
+update_markertime(tims.marker=cook_markertime(time_trunc(newcurr), xin, yin));
 if(tims.lmbcache<0) tims.lmbcache=time_trunc(newcurr);
 
 draw_marker_partitions();
@@ -388,11 +502,12 @@ gdouble x, y, pixstep, offset;
 GdkModifierType state;
 TimeType newcurr;
 int scrolled;
+gint xi, yi;
 
 if(event->is_hint)   
         {
-        gdk_input_window_get_pointer(event->window, event->deviceid,
-                &x, &y, NULL, NULL, NULL, &state);
+	WAVE_GDK_GET_POINTER(event->window, &x, &y, &xi, &yi, &state);
+	WAVE_GDK_GET_POINTER_COPY;
         }
         else
         {
@@ -401,7 +516,6 @@ if(event->is_hint)
         state = event->state;
         }
 
-
 do
 	{
 	scrolled=0;
@@ -480,7 +594,7 @@ do
 	
 	if(state&bmask[in_button_press])
 		{
-		button_motion_common(x,0);
+		button_motion_common(x,y,0);
 		}
 	
 	if(scrolled)	/* make sure counters up top update.. */
@@ -491,8 +605,8 @@ do
 			}
 		}
 
-	gdk_input_window_get_pointer(event->window, event->deviceid,
-		&x, &y, NULL, NULL, NULL, &state);
+	WAVE_GDK_GET_POINTER(event->window, &x, &y, &xi, &yi, &state);
+	WAVE_GDK_GET_POINTER_COPY;
 
 	} while((scrolled)&&(state&bmask[in_button_press]));
 
@@ -506,7 +620,7 @@ if((event->button==1)||((event->button==3)&&(!in_button_press)))
 	in_button_press=event->button;
 	
 	DEBUG(printf("Button Press Event\n"));
-	button_motion_common(event->x,1);
+	button_motion_common(event->x,event->y,1);
 	tims.timecache=tims.start;
 
 	gdk_pointer_grab(widget->window, FALSE,
@@ -526,7 +640,7 @@ if((event->button)&&(event->button==in_button_press))
 	in_button_press=0;
 
 	DEBUG(printf("Button Release Event\n"));
-	button_motion_common(event->x,1);
+	button_motion_common(event->x,event->y,1);
 	tims.timecache=tims.start;
 
 	gdk_pointer_ungrab(event->time);
@@ -1317,6 +1431,8 @@ TimeType tim, rem;
 char timebuff[32];
 char prevover=0;
 
+renderblackout();
+
 tim=tims.start;
 tims.end=tims.start+(((gdouble)wavewidth)*nspx);
 
@@ -1713,7 +1829,7 @@ if(x1>wavewidth)
 	}
 
 /* draw trans */
-type = vtype(t,h->v.vector);
+type = (!(h->flags&(HIST_REAL|HIST_STRING))) ? vtype(t,h->v.vector) : 0;
 if(x0>-1)
 if(use_roundcaps)
 	{
@@ -1969,6 +2085,7 @@ if(x1>wavewidth)
 
 /* draw trans */
 type = vtype2(t,h);
+
 if(x0>-1)
 if(use_roundcaps)
 	{
